# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.

PDF_DATA = {
    # .........User Guide........#
    "introduction_to_coding": {
        "title": "Introduction to Coding",
        "description": "This document shows how the progression through Rapid Router levels cover each stage of the "
        "computing curriculum for key stages one and two.",
        "links": None,
        "url": "user_guide_v1/introduction_to_coding.pdf",
        "page_origin": "#user-guide",
    },
    "levels_guide_1-28": {
        "title": "Levels Guide 1 ‒ 28",
        "description": "Road maps for levels 1 to 28 of Rapid Router. You can find the maps for levels 29 ‒ 109 below.",
        "links": ["levels_guide_29-109"],
        "url": "user_guide_v1/levels_guide_KS1_KS2.pdf",
        "page_origin": "#user-guide",
    },
    "glossary": {
        "title": "Glossary",
        "description": "A glossary of coding terminology, for all the key stages of the curriculum.",
        "links": None,
        "url": "user_guide_v1/glossary.pdf",
        "page_origin": "#user-guide",
    },
    "introduction_to_python": {
        "title": "Introduction to Python in Rapid Router",
        "description": "A summary of the python commands specific to Rapid Router as well as general python statements "
        "students will need to use.",
        "links": None,
        "url": "user_guide_v1/introduction_to_python.pdf",
        "page_origin": "#user-guide",
    },
    "levels_guide_29-109": {
        "title": "Levels Guide 29 ‒ 109",
        "description": "Road maps for levels 29 to 109 of Rapid Router.",
        "links": ["levels_guide_1-28"],
        "url": "user_guide_v1/levels_guide_KS3.pdf",
        "page_origin": "#user-guide",
    },
    "blockly_python_phrasebook": {
        "title": "Blockly ‒ Python Phrasebook",
        "description": "A table showing the conversion from Blockly to Python.",
        "links": None,
        "url": "user_guide_v1/blockly_python_phrasebook.pdf",
        "page_origin": "#user-guide",
    },
    "user_guide_aimmo": {
        "title": "Introduction to Kurono",
        "description": "User guide for Kurono.",
        "links": None,
        "url": "user_guide_v1/kurono_user_guide_v1.pdf",
        "page_origin": "#user-guide",
    },
    # ...................KS1 sessions...........#
    "KS1_session_1": {
        "title": "KS1 ‒ Session 1",
        "description": "Unplugged algorithms for moving along a route",
        "links": ["KS1_S1_1", "KS1_S1_2", "KS1_S1_3", "KS1_S1_4"],
        "url": "KS1_v1/sessions/ks1_s1.pdf",
        "page_origin": "#ks1-sessions",
        "levels_covered": "Print outs",
    },
    "KS1_session_2": {
        "title": "KS1 ‒ Session 2",
        "description": "Starting off on–screen with the app",
        "links": None,
        "url": "KS1_v1/sessions/ks1_s2.pdf",
        "video": "https://www.youtube.com/embed/PQFyF3R6Xts",
        "video_download_link": "videos/KS1_2_Introduction_to_Rapid_Router_with_Matt_and_Chris.mp4",
        "page_origin": "#ks1-sessions",
        "levels_covered": "Levels 1–5",
    },
    "KS1_session_3": {
        "title": "KS1 ‒ Session 3",
        "description": "Creating simple algorithms to reach a single destination",
        "links": ["KS1_S3_1"],
        "url": "KS1_v1/sessions/ks1_s3.pdf",
        "page_origin": "#ks1-sessions",
        "levels_covered": "Levels 6–12",
    },
    "KS1_session_4": {
        "title": "KS1 ‒ Session 4",
        "description": "Creating simple algorithms to reach a single destination along the shortest route",
        "links": ["KS1_S4_1", "KS1_S4_2", "KS1_S4_3", "KS1_S4_4"],
        "url": "KS1_v1/sessions/ks1_s4.pdf",
        "page_origin": "#ks1-sessions",
        "levels_covered": "Levels 13–14",
    },
    "KS1_session_5": {
        "title": "KS1 ‒ Session 5",
        "description": "Create a more complex algorithm to deliver one or more packages on the way",
        "links": ["KS1_S5_1"],
        "url": "KS1_v1/sessions/ks1_s5.pdf",
        "page_origin": "#ks1-sessions",
        "levels_covered": "Levels 14–16",
    },
    # .........KS1 Resource Sheets.........#
    # ...Session 1...#
    "KS1_S1_1": {
        "title": "KS1 ‒ Session 1 ‒ Sheet 1",
        "description": "To be used with Session 1 lesson plan.",
        "links": ["KS1_session_1"],
        "url": "KS1_v1/resource_sheets/ks1_s1_1.pdf",
        "page_origin": "#ks1-resource-sheets",
    },
    "KS1_S1_2": {
        "title": "KS1 ‒ Session 1 ‒ Sheet 2",
        "description": "To be used with Session 1 lesson plan.",
        "links": ["KS1_session_1"],
        "url": "KS1_v1/resource_sheets/ks1_s1_2.pdf",
        "page_origin": "#ks1-resource-sheets",
    },
    "KS1_S1_3": {
        "title": "KS1 ‒ Session 1 ‒ Sheet 3",
        "description": "To be used with Session 1 lesson plan.",
        "links": ["KS1_session_1"],
        "url": "KS1_v1/resource_sheets/ks1_s1_3.pdf",
        "page_origin": "#ks1-resource-sheets",
    },
    "KS1_S1_4": {
        "title": "KS1 ‒ Session 1 ‒ Sheet 4",
        "description": "To be used with Session 1 lesson plan.",
        "links": ["KS1_session_1"],
        "url": "KS1_v1/resource_sheets/ks1_s1_4.pdf",
        "page_origin": "#ks1-resource-sheets",
    },
    # ...Session 3...#
    "KS1_S3_1": {
        "title": "KS1 ‒ Session 3 ‒ Sheet 1",
        "description": "To be used with Session 3 lesson plan.",
        "links": ["KS1_session_3"],
        "url": "KS1_v1/resource_sheets/ks1_s3_1.pdf",
        "page_origin": "#ks1-resource-sheets",
    },
    # ...Session 4...#
    "KS1_S4_1": {
        "title": "KS1 ‒ Session 4 ‒ Sheet 1",
        "description": "To be used with Session 4 lesson plan.",
        "links": ["KS1_session_4"],
        "url": "KS1_v1/resource_sheets/ks1_s4_1.pdf",
        "page_origin": "#ks1-resource-sheets",
    },
    "KS1_S4_2": {
        "title": "KS1 ‒ Session 4 ‒ Sheet 2",
        "description": "To be used with Session 4 lesson plan.",
        "links": ["KS1_session_4"],
        "url": "KS1_v1/resource_sheets/ks1_s4_2.pdf",
        "page_origin": "#ks1-resource-sheets",
    },
    "KS1_S4_3": {
        "title": "KS1 ‒ Session 4 ‒ Sheet 3",
        "description": "To be used with Session 4 lesson plan.",
        "links": ["KS1_session_4"],
        "url": "KS1_v1/resource_sheets/ks1_s4_3.pdf",
        "page_origin": "#ks1-resource-sheets",
    },
    "KS1_S4_4": {
        "title": "KS1 ‒ Session 4 ‒ Sheet 4",
        "description": "To be used with Session 4 lesson plan.",
        "links": ["KS1_session_4"],
        "url": "KS1_v1/resource_sheets/ks1_s4_4.pdf",
        "page_origin": "#ks1-resource-sheets",
    },
    # ...Session 5...#
    "KS1_S5_1": {
        "title": "KS1 ‒ Session 5 ‒ Sheet 1",
        "description": "To be used with Session 5 lesson plan.",
        "links": ["KS1_session_5"],
        "url": "KS1_v1/resource_sheets/ks1_s5_1.pdf",
        "page_origin": "#ks1-resource-sheets",
    },
    # ............KS1 Extras............#
    # ...KS1 Program Solutions...#
    "program_solutions_1-16": {
        "title": "Program Solutions 1 ‒ 16",
        "description": "A table showing the best solutions to levels 1 to 16 of Rapid Router.",
        "links": [
            "program_solutions_17-28",
            "program_solutions_19-43",
            "program_solutions_44-79",
            "program_solutions_80-109",
        ],
        "url": "KS1_v1/ks1_program_solutions.pdf",
        "page_origin": "#ks1-solutions",
    },
    # ...KS1 Activity Set...#
    "KS1_activity_set": {
        "title": "KS1 ‒ Activity Set",
        "description": "This activity set gives you an overview of the Key Stage 1 module. It covers the resources "
        "that will be useful, learning expectations and teacher preparation.",
        "links": None,
        "url": "KS1_v1/ks1_activity_set.pdf",
        "page_origin": "#ks1-activity",
    },
    # ...KS1 Assets...#
    "KS1_assets": {
        "title": "KS1 ‒ Assets",
        "description": "Arrows, scenery and blockly statements that can be cut out for offline activities.",
        "links": ["KS1_code_wall_cards"],
        "url": "KS1_v1/assets/ks1_assets.pdf",
        "page_origin": "#ks1-assets",
    },
    "KS1_code_wall_cards": {
        "title": "KS1 ‒ Code Wall Cards",
        "description": "Keywords that can be displayed in the classroom.",
        "links": ["KS1_assets"],
        "url": "KS1_v1/assets/ks1_code_wall_cards.pdf",
        "page_origin": "#ks1-assets",
    },
    # ...KS1 Assessments...#
    "KS1_assessment_guide": {
        "title": "KS1 ‒ Assessment Guide",
        "description": "Information on how to use the assessment sheets available.",
        "links": ["KS1_self_assessment_sheet", "KS1_teacher_record_sheet"],
        "url": "KS1_v1/assessments/ks1_assessment_sheet.pdf",
        "page_origin": "#ks1-assessments",
    },
    "KS1_self_assessment_sheet": {
        "title": "KS1 ‒ Self Assessment Sheet",
        "description": "Document for students to self assess their understanding of key concepts.",
        "links": ["KS1_assessment_guide", "KS1_teacher_record_sheet"],
        "url": "KS1_v1/assessments/ks1_self_assessment_sheet.pdf",
        "page_origin": "#ks1-assessments",
    },
    "KS1_teacher_record_sheet": {
        "title": "KS1 ‒ Teacher Record Sheet",
        "description": "Document for the teacher to record pupils’ progress.",
        "links": ["KS1_assessment_guide", "KS1_self_assessment_sheet"],
        "url": "KS1_v1/assessments/ks1_teacher_record_sheet.pdf",
        "page_origin": "#ks1-assessments",
    },
    # .........KS2 sessions...........#
    "KS2_session_1": {
        "title": "KS2 ‒ Session 1",
        "description": "Recap: Delivering several packages in a sensible sequence",
        "links": ["KS2_S1_1", "KS2_S1_2"],
        "url": "KS2_v1/sessions/ks2_s1.pdf",
        "page_origin": "#ks2-sessions",
        "levels_covered": "Levels 17–18",
    },
    "KS2_session_2": {
        "title": "KS2 ‒ Session 2",
        "description": "Introducing the repeat loop",
        "links": ["KS2_S2_1", "KS2_S2_2", "KS2_S2_3"],
        "url": "KS2_v1/sessions/ks2_s2.pdf",
        "video": "https://www.youtube.com/embed/vFGd0v3msRE",
        "video_download_link": "videos/KS1_7_Mahana_explains_repeat_functions.mp4",
        "page_origin": "#ks2-sessions",
        "levels_covered": "Levels 19–22",
    },
    "KS2_session_3": {
        "title": "KS2 ‒ Session 3",
        "description": "Follow–on and extension work on multiple repeat loops and nested repeats",
        "links": ["KS2_S3_1", "KS2_S3_2"],
        "url": "KS2_v1/sessions/ks2_s3.pdf",
        "page_origin": "#ks2-sessions",
        "levels_covered": "Levels 23–25",
    },
    "KS2_session_4": {
        "title": "KS2 ‒ Session 4",
        "description": "Students create their own routes",
        "links": ["KS2_S4_1"],
        "url": "KS2_v1/sessions/ks2_s4.pdf",
        "page_origin": "#ks2-sessions",
        "levels_covered": "Levels 26–28",
    },
    "KS2_session_5": {
        "title": "KS2 ‒ Session 5",
        "description": "Try out a partner’s route",
        "links": None,
        "url": "KS2_v1/sessions/ks2_s5.pdf",
        "page_origin": "#ks2-sessions",
        "levels_covered": "",
    },
    # .........KS2 Resource Sheets.........#
    # ...Session 1 ...#
    "KS2_S1_1": {
        "title": "KS2 ‒ Session 1 ‒ Sheet 1",
        "description": "To be used with Session 1 lesson plan.",
        "links": ["KS2_session_1"],
        "url": "KS2_v1/resource_sheets/ks2_s1_1.pdf",
        "page_origin": "#ks2-resource-sheets",
    },
    "KS2_S1_2": {
        "title": "KS2 ‒ Session 1 ‒ Sheet 2",
        "description": "To be used with Session 1 lesson plan.",
        "links": ["KS2_session_1"],
        "url": "KS2_v1/resource_sheets/ks2_s1_2.pdf",
        "page_origin": "#ks2-resource-sheets",
    },
    # ...Session 2...#
    "KS2_S2_1": {
        "title": "KS2 ‒ Session 2 ‒ Sheet 1",
        "description": "To be used with Session 2 lesson plan.",
        "links": ["KS2_session_2"],
        "url": "KS2_v1/resource_sheets/ks2_s2_1.pdf",
        "page_origin": "#ks2-resource-sheets",
    },
    "KS2_S2_2": {
        "title": "KS2 ‒ Session 2 ‒ Sheet 2",
        "description": "To be used with Session 2 lesson plan.",
        "links": ["KS2_session_2"],
        "url": "KS2_v1/resource_sheets/ks2_s2_2.pdf",
        "page_origin": "#ks2-resource-sheets",
    },
    "KS2_S2_3": {
        "title": "KS2 ‒ Session 2 ‒ Sheet 3",
        "description": "To be used with Session 2 lesson plan.",
        "links": ["KS2_session_2"],
        "url": "KS2_v1/resource_sheets/ks2_s2_3.pdf",
        "page_origin": "#ks2-resource-sheets",
    },
    # ...Session 3...#
    "KS2_S3_1": {
        "title": "KS2 ‒ Session 3 ‒ Sheet 1",
        "description": "To be used with Session 3 lesson plan.",
        "links": ["KS2_session_3"],
        "url": "KS2_v1/resource_sheets/ks2_s3_1.pdf",
        "page_origin": "#ks2-resource-sheets",
    },
    "KS2_S3_2": {
        "title": "KS2 ‒ Session 3 ‒ Sheet 2",
        "description": "To be used with Session 3 lesson plan.",
        "links": ["KS2_session_3"],
        "url": "KS2_v1/resource_sheets/ks2_s3_2.pdf",
        "page_origin": "#ks2-resource-sheets",
    },
    # ...Session 4...#
    "KS2_S4_1": {
        "title": "KS2 ‒ Session 4 ‒ Sheet 1",
        "description": "To be used with Session 4 lesson plan.",
        "links": ["KS2_session_4"],
        "url": "KS2_v1/resource_sheets/ks2_s4_1.pdf",
        "page_origin": "#ks2-resource-sheets",
    },
    # ............KS2 Extras............#
    # ...KS2 Program Solutions...#
    "program_solutions_17-28": {
        "title": "Program Solutions 17 ‒ 28",
        "description": "A table showing the best solutions to levels 17 to 28 of Rapid Router.",
        "links": [
            "program_solutions_1-16",
            "program_solutions_19-43",
            "program_solutions_44-79",
            "program_solutions_80-109",
        ],
        "url": "KS2_v1/ks2_program_solutions.pdf",
        "page_origin": "#ks2-solutions",
    },
    # ...KS2 Activity Set...#
    "KS2_activity_set": {
        "title": "KS2 ‒ Activity Set",
        "description": "This activity set gives you an overview of the Key Stage 2 module. It covers the "
        "resources that will be useful, learning expectations and teacher preparation.",
        "links": None,
        "url": "KS2_v1/ks2_activity_set.pdf",
        "page_origin": "#ks2-activity",
    },
    # ...KS2 Assets...#
    "KS2_assets": {
        "title": "KS2 ‒ Assets",
        "description": "Blockly statements for Key Stage 2 students that can be cut out for offline activities.",
        "links": ["KS2_code_wall_cards"],
        "url": "KS2_v1/assets/ks2_assets.pdf",
        "page_origin": "#ks2-assets",
    },
    "KS2_code_wall_cards": {
        "title": "KS2 ‒ Code Wall Cards",
        "description": "Keywords that can be displayed in the classroom.",
        "links": ["KS2_assets"],
        "url": "KS2_v1/assets/ks2_code_wall_cards.pdf",
        "page_origin": "#ks2-assets",
    },
    # ...KS2 Assessments...#
    "KS2_assessment_guide": {
        "title": "KS2 ‒ Assessment Guide",
        "description": "Information on how to use the assessment sheets available.",
        "links": ["KS2_self_assessment_sheet", "KS2_teacher_record_sheet"],
        "url": "KS2_v1/assessments/ks2_assessment_sheet.pdf",
        "page_origin": "#ks2-assessments",
    },
    "KS2_self_assessment_sheet": {
        "title": "KS2 ‒ Self Assessment Sheet",
        "description": "Document for students to self assess their understanding of key concepts.",
        "links": ["KS2_assessment_guide", "KS2_teacher_record_sheet"],
        "url": "KS2_v1/assessments/ks2_self_assessment_sheet.pdf",
        "page_origin": "#ks2-assessments",
    },
    "KS2_teacher_record_sheet": {
        "title": "KS2 ‒ Teacher Record Sheet",
        "description": "Document for the teacher to record pupils’ progress.",
        "links": ["KS2_assessment_guide", "KS2_self_assessment_sheet"],
        "url": "KS2_v1/assessments/ks2_teacher_record_sheet.pdf",
        "page_origin": "#ks2-assessments",
    },
    # .........LKS3 sessions...........#
    "Lower_KS3_session_1": {
        "title": "LKS3 ‒ Session 1",
        "description": "Recap: Using a simple ‘repeat’ loop",
        "links": ["Lower_KS3_S1_1"],
        "url": "KS3_v1/sessions/ks3_s1.pdf",
        "page_origin": "#lks3-sessions",
        "levels_covered": "Levels 19–28",
    },
    "Lower_KS3_session_2": {
        "title": "LKS3 ‒ Session 2",
        "description": "Using repeat with a condition on a simple route: ‘repeat until’",
        "links": ["Lower_KS3_S2_1", "Lower_KS3_S2_2"],
        "url": "KS3_v1/sessions/ks3_s2.pdf",
        "video": "https://www.youtube.com/embed/EDwc80X_LQI",
        "video_download_link": "videos/LKS2_2_Ana_discusses_repeat_until_and_repeat_while_functions.mp4",
        "page_origin": "#lks3-sessions",
        "levels_covered": "Levels 29–32",
    },
    "Lower_KS3_session_3": {
        "title": "LKS3 ‒ Session 3",
        "description": "Using simple selection and repetition: Repeat until at destination ‘if…do’",
        "links": ["Lower_KS3_S3_1"],
        "url": "KS3_v1/sessions/ks3_s3.pdf",
        "video": "https://www.youtube.com/embed/O0RXbJyYq8o",
        "video_download_link": "videos/LKS2_3_Chris_and_Matt_introduce_if_statements.mp4",
        "page_origin": "#lks3-sessions",
        "levels_covered": "Levels 33–35",
    },
    "Lower_KS3_session_4": {
        "title": "LKS3 ‒ Session 4",
        "description": "General solution using repeat until and if…do… (including else if for more able students)",
        "links": [
            "Lower_KS3_S4_1",
            "Lower_KS3_S4_2",
            "Lower_KS3_S4_3",
            "Lower_KS3_S4_4",
        ],
        "url": "KS3_v1/sessions/ks3_s4.pdf",
        "video": "https://www.youtube.com/embed/GUUJSRuAyU0",
        "video_download_link": "videos/LKS2_4_Helen_explains_if_else_statements.mp4",
        "page_origin": "#lks3-sessions",
        "levels_covered": "Levels 36–38",
    },
    "Lower_KS3_session_5": {
        "title": "LKS3 ‒ Session 5",
        "description": "Group extension ‒ Explain why the general program does not work for some more complex maps",
        "links": ["Lower_KS3_S5_1"],
        "url": "KS3_v1/sessions/ks3_s5.pdf",
        "page_origin": "#lks3-sessions",
        "levels_covered": "Levels 39–43",
    },
    # .........IKS3 sessions...........#
    "Intermediate_KS3_session_6": {
        "title": "IKS3 ‒ Session 6",
        "description": "Traffic lights: understanding variables and the if do…if else statements",
        "links": ["Intermediate_KS3_S6_1"],
        "url": "KS3_v1/sessions/ks3_s6.pdf",
        "video": "https://www.youtube.com/embed/UFpsEjaB_DI",
        "video_download_link": "videos/LKS2_6_Will_discusses_variables.mp4",
        "page_origin": "#iks3-sessions",
        "levels_covered": "Levels 44–47",
    },
    "Intermediate_KS3_session_7": {
        "title": "IKS3 ‒ Session 7",
        "description": "Applying their programming knowledge to create a game challenge",
        "links": ["Intermediate_KS3_S7_1", "Intermediate_KS3_S7_2"],
        "url": "KS3_v1/sessions/ks3_s7.pdf",
        "video": "https://www.youtube.com/embed/vFGd0v3msRE",
        "video_download_link": "videos/KS1_7_Mahana_explains_repeat_functions.mp4",
        "page_origin": "#iks3-sessions",
        "levels_covered": "Level creator",
    },
    "Intermediate_KS3_session_8": {
        "title": "IKS3 ‒ Session 8",
        "description": "Applying their programming knowledge to play and evaluate a game challenge",
        "links": ["Intermediate_KS3_self_assessment_sheet"],
        "url": "KS3_v1/sessions/ks3_s8.pdf",
        "page_origin": "#iks3-sessions",
        "levels_covered": "Shared level",
    },
    "Intermediate_KS3_session_9": {
        "title": "IKS3 ‒ Session 9",
        "description": "What do we already know? (Recap the Blockly commands previously encountered)",
        "links": ["Intermediate_KS3_S9_1"],
        "url": "KS3_v1/sessions/ks3_s9.pdf",
        "page_origin": "#iks3-sessions",
        "levels_covered": "Levels 51–60",
    },
    "Intermediate_KS3_session_10": {
        "title": "IKS3 ‒ Session 10",
        "description": "Breaking down the problem in chunks (understanding procedures)",
        "links": ["Intermediate_KS3_S10_1", "Intermediate_KS3_S10_2"],
        "url": "KS3_v1/sessions/ks3_s10.pdf",
        "video": "https://www.youtube.com/embed/zsWwscJrEhs",
        "video_download_link": "videos/LKS3_Ed_Armitage_explains_functions_procedures.mp4",
        "page_origin": "#iks3-sessions",
        "levels_covered": "Levels 61–67",
    },
    # ........UKS3 sessions...........#
    "Upper_KS3_session_11": {
        "title": "UKS3 ‒ Session 11",
        "description": "Switching from Blockly to Python",
        "links": ["Upper_KS3_S11_1", "Upper_KS3_S11_2"],
        "url": "KS3_v1/sessions/ks3_s11.pdf",
        "video": "https://www.youtube.com/embed/5atj2caRjyA",
        "video_download_link": "videos/LKS3_Kenny_introduces_real_programming_language_Python.mp4",
        "page_origin": "#uks3-sessions",
        "levels_covered": "Levels 80–84",
    },
    "Upper_KS3_session_12": {
        "title": "UKS3 ‒ Session 12",
        "description": "Understanding more Python commands (while, if…elif…else)",
        "links": ["Upper_KS3_S12_1"],
        "url": "KS3_v1/sessions/ks3_s12.pdf",
        "page_origin": "#uks3-sessions",
        "levels_covered": "Levels 85–91",
    },
    "Upper_KS3_session_13": {
        "title": "UKS3 ‒ Session 13",
        "description": "Writing basic code directly in Python (forwards, turn, print, repetition)",
        "links": ["Upper_KS3_S13_1", "Upper_KS3_S13_2", "Upper_KS3_S13_3"],
        "url": "KS3_v1/sessions/ks3_s13.pdf",
        "page_origin": "#uks3-sessions",
        "levels_covered": "Levels 92–97",
    },
    "Upper_KS3_session_14": {
        "title": "UKS3 ‒ Session 14",
        "description": "Flying Solo with Python! Programming independently using selection, defining "
        "procedures/functions",
        "links": ["Upper_KS3_S14_1", "Upper_KS3_S14_2"],
        "url": "KS3_v1/sessions/ks3_s14.pdf",
        "page_origin": "#uks3-sessions",
        "levels_covered": "Levels 98–105",
    },
    "Upper_KS3_session_15": {
        "title": "UKS3 ‒ Session 15",
        "description": "Increase the challenge ‒ creating new Python variables, incrementing variables",
        "links": ["Upper_KS3_S15_1", "Upper_KS3_S15_2"],
        "url": "KS3_v1/sessions/ks3_s15.pdf",
        "page_origin": "#uks3-sessions",
        "levels_covered": "Levels 106–109",
    },
    "Upper_KS3_session_16": {
        "title": "UKS3 ‒ Session 16",
        "description": "Assessment session ‒ applying your coding knowledge to create a game challenge for a partner",
        "links": ["Upper_KS3_S16_1", "Upper_KS3_S16_2"],
        "url": "KS3_v1/sessions/ks3_s16.pdf",
        "page_origin": "#uks3-sessions",
        "levels_covered": "Level creator",
    },
    # .........LKS3 Resource Sheets.........#
    # ...Session 1...#
    "Lower_KS3_S1_1": {
        "title": "LKS3 ‒ Session 1 ‒ Sheet 1",
        "description": "To be used with Session 1 lesson plan.",
        "links": ["Lower_KS3_session_1"],
        "url": "KS3_v1/resource_sheets/ks3_s1_1.pdf",
        "page_origin": "#lks3-resource-sheets",
    },
    # ... Session 2 ... #
    "Lower_KS3_S2_1": {
        "title": "LKS3 ‒ Session 2 ‒ Sheet 1",
        "description": "To be used with Session 2 lesson plan.",
        "links": ["Lower_KS3_session_2"],
        "url": "KS3_v1/resource_sheets/ks3_s2_1.pdf",
        "page_origin": "#lks3-resource-sheets",
    },
    "Lower_KS3_S2_2": {
        "title": "LKS3 ‒ Session 2 ‒ Sheet 2",
        "description": "To be used with Session 2 lesson plan.",
        "links": ["Lower_KS3_session_2"],
        "url": "KS3_v1/resource_sheets/ks3_s2_2.pdf",
        "page_origin": "#lks3-resource-sheets",
    },
    # ...Session 3...#
    "Lower_KS3_S3_1": {
        "title": "LKS3 ‒ Session 3 ‒ Sheet 1",
        "description": "To be used with Session 3 lesson plan.",
        "links": ["Lower_KS3_session_3"],
        "url": "KS3_v1/resource_sheets/ks3_s3_1.pdf",
        "page_origin": "#lks3-resource-sheets",
    },
    # ...Session 4...#
    "Lower_KS3_S4_1": {
        "title": "LKS3 ‒ Session 4 ‒ Sheet 1",
        "description": "To be used with Session 4 lesson plan.",
        "links": ["Lower_KS3_session_4"],
        "url": "KS3_v1/resource_sheets/ks3_s4_1.pdf",
        "page_origin": "#lks3-resource-sheets",
    },
    "Lower_KS3_S4_2": {
        "title": "LKS3 ‒ Session 4 ‒ Sheet 2",
        "description": "To be used with Session 4 lesson plan.",
        "links": ["Lower_KS3_session_4"],
        "url": "KS3_v1/resource_sheets/ks3_s4_2.pdf",
        "page_origin": "#lks3-resource-sheets",
    },
    "Lower_KS3_S4_3": {
        "title": "LKS3 ‒ Session 4 ‒ Sheet 3",
        "description": "To be used with Session 4 lesson plan.",
        "links": ["Lower_KS3_session_4"],
        "url": "KS3_v1/resource_sheets/ks3_s4_3.pdf",
        "page_origin": "#lks3-resource-sheets",
    },
    "Lower_KS3_S4_4": {
        "title": "LKS3 ‒ Session 4 ‒ Sheet 4",
        "description": "To be used with Session 4 lesson plan.",
        "links": ["Lower_KS3_session_4"],
        "url": "KS3_v1/resource_sheets/ks3_s4_4.pdf",
        "page_origin": "#lks3-resource-sheets",
    },
    # ...Session 5...#
    "Lower_KS3_S5_1": {
        "title": "LKS3 ‒ Session 5 ‒ Sheet 1",
        "description": "To be used with Session 5 lesson plan.",
        "links": ["Lower_KS3_session_5"],
        "url": "KS3_v1/resource_sheets/ks3_s5_1.pdf",
        "page_origin": "#lks3-resource-sheets",
    },
    # .........IKS3 Resource Sheets.........#
    # ...Session 6...#
    "Intermediate_KS3_S6_1": {
        "title": "IKS3 ‒ Session 6 ‒ Sheet 1",
        "description": "To be used with Session 6 lesson plan.",
        "links": ["Intermediate_KS3_session_6"],
        "url": "KS3_v1/resource_sheets/ks3_s6_1.pdf",
        "page_origin": "#iks3-resource-sheets",
    },
    "Intermediate_KS3_S6_2": {
        "title": "IKS3 ‒ Session 6 ‒ Sheet 2",
        "description": "To be used with Session 6 lesson plan.",
        "links": ["Intermediate_KS3_session_6"],
        "url": "KS3_v1/resource_sheets/ks3_s6_2.pdf",
        "page_origin": "#iks3-resource-sheets",
    },
    # ...Session 7...#
    "Intermediate_KS3_S7_1": {
        "title": "IKS3 ‒ Session 7 ‒ Sheet 1",
        "description": "To be used with Session 7 lesson plan.",
        "links": ["Intermediate_KS3_session_7"],
        "url": "KS3_v1/resource_sheets/ks3_s7_1.pdf",
        "page_origin": "#iks3-resource-sheets",
    },
    "Intermediate_KS3_S7_2": {
        "title": "IKS3 ‒ Session 7 ‒ Sheet 2",
        "description": "To be used with Session 7 lesson plan.",
        "links": ["Intermediate_KS3_session_7"],
        "url": "KS3_v1/resource_sheets/ks3_s7_2.pdf",
        "page_origin": "#iks3-resource-sheets",
    },
    # ...Session 9...#
    "Intermediate_KS3_S9_1": {
        "title": "IKS3 ‒ Session 9 ‒ Sheet 1",
        "description": "To be used with Session 9 lesson plan.",
        "links": ["Intermediate_KS3_session_9"],
        "url": "KS3_v1/resource_sheets/ks3_s9_1.pdf",
        "page_origin": "#iks3-resource-sheets",
    },
    "Intermediate_KS3_S9_2": {
        "title": "IKS3 ‒ Session 9 ‒ Sheet 2",
        "description": "To be used with Session 9 lesson plan.",
        "links": ["Intermediate_KS3_session_9"],
        "url": "KS3_v1/resource_sheets/ks3_s9_2.pdf",
        "page_origin": "#iks3-resource-sheets",
    },
    # ...Session 10...#
    "Intermediate_KS3_S10_1": {
        "title": "IKS3 ‒ Session 10 ‒ Sheet 1",
        "description": "To be used with Session 10 lesson plan.",
        "links": ["Intermediate_KS3_session_10"],
        "url": "KS3_v1/resource_sheets/ks3_s10_1.pdf",
        "page_origin": "#iks3-resource-sheets",
    },
    "Intermediate_KS3_S10_2": {
        "title": "IKS3 ‒ Session 10 ‒ Sheet 2",
        "description": "To be used with Session 10 lesson plan.",
        "links": ["Intermediate_KS3_session_10"],
        "url": "KS3_v1/resource_sheets/ks3_s10_2.pdf",
        "page_origin": "#iks3-resource-sheets",
    },
    # .........UKS3 Resource Sheets.........#
    # ...Session 11...#
    "Upper_KS3_S11_1": {
        "title": "UKS3 ‒ Session 11 ‒ Sheet 1",
        "description": "To be used with Session 11 lesson plan.",
        "links": ["Upper_KS3_session_11"],
        "url": "KS3_v1/resource_sheets/ks3_s11_1.pdf",
        "page_origin": "#uks3-resource-sheets",
    },
    "Upper_KS3_S11_2": {
        "title": "UKS3 ‒ Session 11 ‒ Sheet 2",
        "description": "To be used with Session 11 lesson plan.",
        "links": ["Upper_KS3_session_11"],
        "url": "KS3_v1/resource_sheets/ks3_s11_2.pdf",
        "page_origin": "#uks3-resource-sheets",
    },
    # ...Session 12...#
    "Upper_KS3_S12_1": {
        "title": "UKS3 ‒ Session 12 ‒ Sheet 1",
        "description": "To be used with Session 12 lesson plan.",
        "links": ["Upper_KS3_session_12"],
        "url": "KS3_v1/resource_sheets/ks3_s12_1.pdf",
        "page_origin": "#uks3-resource-sheets",
    },
    # ...Session 13...#
    "Upper_KS3_S13_1": {
        "title": "UKS3 ‒ Session 13 ‒ Sheet 1",
        "description": "To be used with Session 13 lesson plan.",
        "links": ["Upper_KS3_session_13"],
        "url": "KS3_v1/resource_sheets/ks3_s13_1.pdf",
        "page_origin": "#uks3-resource-sheets",
    },
    "Upper_KS3_S13_2": {
        "title": "UKS3 ‒ Session 13 ‒ Sheet 2",
        "description": "To be used with Session 13 lesson plan.",
        "links": ["Upper_KS3_session_13"],
        "url": "KS3_v1/resource_sheets/ks3_s13_2.pdf",
        "page_origin": "#uks3-resource-sheets",
    },
    "Upper_KS3_S13_3": {
        "title": "UKS3 ‒ Session 13 ‒ Sheet 3",
        "description": "To be used with Session 13 lesson plan.",
        "links": ["Upper_KS3_session_13"],
        "url": "KS3_v1/resource_sheets/ks3_s13_3.pdf",
        "page_origin": "#uks3-resource-sheets",
    },
    # ...Session 14...#
    "Upper_KS3_S14_1": {
        "title": "UKS3 ‒ Session 14 ‒ Sheet 1",
        "description": "To be used with Session 14 lesson plan.",
        "links": ["Upper_KS3_session_14"],
        "url": "KS3_v1/resource_sheets/ks3_s14_1.pdf",
        "page_origin": "#uks3-resource-sheets",
    },
    # ...Session 15...#
    "Upper_KS3_S15_1": {
        "title": "UKS3 ‒ Session 15 ‒ Sheet 1",
        "description": "To be used with Session 15 lesson plan.",
        "links": ["Upper_KS3_session_15"],
        "url": "KS3_v1/resource_sheets/ks3_s15_1.pdf",
        "page_origin": "#uks3-resource-sheets",
    },
    "Upper_KS3_S15_2": {
        "title": "UKS3 ‒ Session 15 ‒ Sheet 2",
        "description": "To be used with Session 15 lesson plan.",
        "links": ["Upper_KS3_session_15"],
        "url": "KS3_v1/resource_sheets/ks3_s15_2.pdf",
        "page_origin": "#uks3-resource-sheets",
    },
    # ...Session 16...#
    "Upper_KS3_S16_1": {
        "title": "UKS3 ‒ Session 16 ‒ Sheet 1",
        "description": "To be used with Session 16 lesson plan.",
        "links": ["Upper_KS3_session_16"],
        "url": "KS3_v1/resource_sheets/ks3_s16_1.pdf",
        "page_origin": "#uks3-resource-sheets",
    },
    "Upper_KS3_S16_2": {
        "title": "UKS3 ‒ Session 16 ‒ Sheet 2",
        "description": "To be used with Session 16 lesson plan.",
        "links": ["Upper_KS3_session_16"],
        "url": "KS3_v1/resource_sheets/ks3_s16_2.pdf",
        "page_origin": "#uks3-resource-sheets",
    },
    # ............KS3 Extras............#
    # ...KS3 Program Solutions...#
    "program_solutions_19-43": {
        "title": "Program Solutions 19 ‒ 43",
        "description": "A table showing the best solutions to levels 19 to 43 of Rapid Router.",
        "links": [
            "program_solutions_1-16",
            "program_solutions_17-28",
            "program_solutions_44-79",
            "program_solutions_80-109",
        ],
        "url": "KS3_v1/program_solutions/lks3_program_solutions.pdf",
        "page_origin": "#ks3-solutions",
    },
    "program_solutions_44-79": {
        "title": "Program Solutions 44 ‒ 79",
        "description": "A table showing the best solutions to levels 44 to 79 of Rapid Router.",
        "links": [
            "program_solutions_1-16",
            "program_solutions_17-28",
            "program_solutions_19-43",
            "program_solutions_80-109",
        ],
        "url": "KS3_v1/program_solutions/iks3_program_solutions.pdf",
        "page_origin": "#ks3-solutions",
    },
    "program_solutions_80-109": {
        "title": "Program Solutions 80 ‒ 109",
        "description": "A table showing the best solutions to levels 80 to 109 of Rapid Router.",
        "links": [
            "program_solutions_1-16",
            "program_solutions_17-28",
            "program_solutions_19-43",
            "program_solutions_44-79",
        ],
        "url": "KS3_v1/program_solutions/uks3_program_solutions.pdf",
        "page_origin": "#ks3-solutions",
    },
    # ...KS3 Activity Sets...#
    "Lower_KS3_activity_set": {
        "title": "Lower KS3 ‒ Activity Set",
        "description": "This activity set gives you an overview of the Lower Key Stage 3 module. It covers the "
        "resources that will be useful, learning expectations and teacher preparation.",
        "links": ["Intermediate_KS3_activity_set", "Upper_KS3_activity_set"],
        "url": "KS3_v1/activity_sets/lks3_activity_set.pdf",
        "page_origin": "#ks3-activity",
    },
    "Intermediate_KS3_activity_set": {
        "title": "Intermediate KS3 ‒ Activity Set",
        "description": "This activity set gives you an overview of the Intermediate Key Stage 3 module. It covers the "
        "resources that will be useful, learning expectations and teacher preparation.",
        "links": ["Lower_KS3_activity_set", "Upper_KS3_activity_set"],
        "url": "KS3_v1/activity_sets/iks3_activity_set.pdf",
        "page_origin": "#ks3-activity",
    },
    "Upper_KS3_activity_set": {
        "title": "Upper KS3 ‒ Activity Set",
        "description": "This activity set gives you an overview of the Upper Key Stage 3 module. It covers the "
        "resources that will be useful, learning expectations and teacher preparation.",
        "links": ["Lower_KS3_activity_set", "Intermediate_KS3_activity_set"],
        "url": "KS3_v1/activity_sets/uks3_activity_set.pdf",
        "page_origin": "#ks3-activity",
    },
    # ...KS3 Assets...#
    "Lower_KS3_assets": {
        "title": "Lower KS3 ‒ Assets",
        "description": "Blockly and Python statements for Lower Key Stage 3 students that can be cut out for offline "
        "activities.",
        "links": ["Intermediate_KS3_assets", "Upper_KS3_assets"],
        "url": "KS3_v1/assets/lks3_assets.pdf",
        "page_origin": "#ks3-assets",
    },
    "Intermediate_KS3_assets": {
        "title": "Intermediate KS3 ‒ Assets",
        "description": "Blockly and Python statements for Intermediate Key Stage 3 students that can be cut out for "
        "offline activities.",
        "links": ["Lower_KS3_assets", "Upper_KS3_assets"],
        "url": "KS3_v1/assets/iks3_assets.pdf",
        "page_origin": "#ks3-assets",
    },
    "Upper_KS3_assets": {
        "title": "Upper KS3 ‒ Assets",
        "description": "Blockly and Python statements for Upper Key Stage 3 students that can be cut out for offline "
        "activities.",
        "links": ["Lower_KS3_assets", "Intermediate_KS3_assets"],
        "url": "KS3_v1/assets/uks3_assets.pdf",
        "page_origin": "#ks3-assets",
    },
    # ...KS3 Assessments...#
    "Lower_KS3_assessment_sheet": {
        "title": "Lower KS3 ‒ Assessment Guide",
        "description": "Information on how to use the assessment sheets available.",
        "links": ["Intermediate_KS3_assessment_sheet", "Upper_KS3_assessment_sheet"],
        "url": "KS3_v1/assessments/lks3_assessment_sheet.pdf",
        "page_origin": "#ks3-assessments",
    },
    "Lower_KS3_self_assessment_sheet": {
        "title": "Lower KS3 ‒ Self Assessment Sheet",
        "description": "Document for students to self assess their understanding of key concepts.",
        "links": [
            "Intermediate_KS3_self_assessment_sheet",
            "Upper_KS3_self_assessment_sheet",
        ],
        "url": "KS3_v1/assessments/lks3_self_assessment_sheet.pdf",
        "page_origin": "#ks3-assessments",
    },
    "Lower_KS3_teacher_record_sheet": {
        "title": "Lower KS3 ‒ Teacher Record Sheet",
        "description": "Document to record students' progress.",
        "links": [
            "Intermediate_KS3_teacher_record_sheet",
            "Upper_KS3_teacher_record_sheet",
        ],
        "url": "KS3_v1/assessments/lks3_teacher_record_sheet.pdf",
        "page_origin": "#ks3-assessments",
    },
    "Intermediate_KS3_assessment_sheet": {
        "title": "Intermediate KS3 ‒ Assessment Guide",
        "description": "Information on how to use the assessment sheets available.",
        "links": ["Lower_KS3_assessment_sheet", "Upper_KS3_assessment_sheet"],
        "url": "KS3_v1/assessments/iks3_assessment_sheet.pdf",
        "page_origin": "#ks3-assessments",
    },
    "Intermediate_KS3_self_assessment_sheet": {
        "title": "Intermediate KS3 ‒ Self Assessment Guide",
        "description": "Document for students to self assess their understanding of key concepts.",
        "links": ["Lower_KS3_self_assessment_sheet", "Upper_KS3_self_assessment_sheet"],
        "url": "KS3_v1/assessments/iks3_self_assessment_sheet.pdf",
        "page_origin": "#ks3-assessments",
    },
    "Intermediate_KS3_teacher_record_sheet": {
        "title": "Intermediate KS3 ‒ Teacher Record Sheet",
        "description": "Document to record students' progress.",
        "links": ["Lower_KS3_teacher_record_sheet", "Upper_KS3_teacher_record_sheet"],
        "url": "KS3_v1/assessments/iks3_teacher_record_sheet.pdf",
        "page_origin": "#ks3-assessments",
    },
    "Upper_KS3_assessment_sheet": {
        "title": "Upper KS3 ‒ Assessment Guide",
        "description": "Information on how to use the assessment sheets available.",
        "links": ["Lower_KS3_assessment_sheet", "Intermediate_KS3_assessment_sheet"],
        "url": "KS3_v1/assessments/uks3_assessment_sheet.pdf",
        "page_origin": "#ks3-assessments",
    },
    "Upper_KS3_self_assessment_sheet": {
        "title": "Upper KS3 ‒ Self Assessment Guide",
        "description": "Information on how to use the assessment sheets available.",
        "links": [
            "Lower_KS3_self_assessment_sheet",
            "Intermediate_KS3_self_assessment_sheet",
        ],
        "url": "KS3_v1/assessments/uks3_self_assessment_sheet.pdf",
        "page_origin": "#ks3-assessments",
    },
    "Upper_KS3_teacher_record_sheet": {
        "title": "Upper KS3 ‒ Teacher Record Sheet",
        "description": "Document to record students' progress.",
        "links": [
            "Lower_KS3_teacher_record_sheet",
            "Intermediate_KS3_teacher_record_sheet",
        ],
        "url": "KS3_v1/assessments/uks3_teacher_record_sheet.pdf",
        "page_origin": "#ks3-assessments",
    },
    # ...Kurono...#
    # ...Teacher Guides...#
    "Kurono_teacher_guide_1": {
        "title": "Teacher Guide 1",
        "description": "Challenge 1",
        "links": ["Kurono_challenge_1", "Kurono_teacher_guide_2"],
        "url": "kurono_v2/teacher_guides/teacher_guide_S1_v1.pdf",
        "page_origin": "#ks3-sessions",
    },
    "Kurono_teacher_guide_2": {
        "title": "Teacher Guide 2",
        "description": "Challenge 2",
        "links": ["Kurono_challenge_2", "Kurono_teacher_guide_1"],
        "url": "kurono_v2/teacher_guides/teacher_guide_S2_v1.pdf",
        "page_origin": "#ks3-sessions",
    },
    # ...Resource Sheets...#
    "Kurono_API_guide": {
        "title": "Kurono API guide",
        "description": "To be used to code the Kurono game.",
        "links": [],
        "url": "kurono_v2/resource_sheets/kurono_API_guide_v1.pdf",
        "page_origin": "#ks3-resource-sheets",
    },
    "Kurono_user_guide": {
        "title": "Kurono User Guide",
        "description": "",
        "links": [],
        "url": "kurono_v2/resource_sheets/user_guide_v1.pdf",
        "page_origin": "#ks3-resource-sheets",
    },
    # ...Intermediate Python Challenges...#
    "Kurono_challenge_1": {
        "title": "Student Challenge 1",
        "description": "Move your avatar",
        "links": ["Kurono_challenge_2"],
        "url": "kurono_v2/student_challenges/SC1_v1.pdf",
        "page_origin": "#ks3-challenges",
    },
    "Kurono_challenge_2": {
        "title": "Student Challenge 2",
        "description": "Obstacles and Artefacts",
        "links": ["Kurono_challenge_1"],
        "url": "kurono_v2/student_challenges/SC2_v1.pdf",
        "page_origin": "#ks3-challenges",
    },
}



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from common.models import Teacher, School


def generate_details(**kwargs):
    name = kwargs.get("name", "School %d" % generate_details.next_id)
    postcode = kwargs.get("postcode", "Al10 9NE")

    generate_details.next_id += 1

    return name, postcode


generate_details.next_id = 1


def create_organisation_directly(teacher_email, **kwargs):
    name, postcode = generate_details(**kwargs)

    school = School.objects.create(
        name=name, postcode=postcode, country="GB", town="", latitude="", longitude=""
    )

    teacher = Teacher.objects.get(new_user__email=teacher_email)
    teacher.school = school
    teacher.is_admin = True
    teacher.save()

    return name, postcode


def join_teacher_to_organisation(teacher_email, org_name, postcode, is_admin=False):
    teacher = Teacher.objects.get(new_user__email=teacher_email)
    school = School.objects.get(name=org_name, postcode=postcode)

    teacher.school = school
    teacher.is_admin = is_admin
    teacher.save()


def create_organisation(page, password):

    name, postcode = generate_details()
    page = page.create_organisation(name, password, postcode)

    return page, name, postcode



# coding=utf-8
MATERIALS_BANNER = {
    "title": "Teaching Packs",
    "subtitle": "",
    "text": "We’ve created a comprehensive set of teaching materials to give you the "
    "confidence and resources to use Code for Life’s games to help you teach students "
    "the English National Computing Curriculum right through their school life.",
    "image_class": "banner--picture--tools",
}



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
import requests

try:
    from exceptions import Exception, ValueError
except ImportError:
    pass


class RequestException(Exception):
    pass


class ApiException(Exception):
    pass


def is_GB(component):
    return "country" in component["types"] and component["short_name"] == "GB"


def extract_locality(components):
    town = None
    country = None

    for component in components:
        if "locality" in component["types"]:
            town = component["long_name"]

        if "postal_town" in component["types"]:
            town = component["long_name"]

        if "country" in component["types"]:
            country = component["short_name"]

    return town, country


def extract_location_data(results):
    for result in results:
        town, country = extract_locality(result["address_components"])
        location = result["geometry"]["location"]
        return country, town, location["lat"], location["lng"]

    return None, None, None, None


# Uses Google Maps API to lookup location data using postcode + country(ISO 3166-1 alpha-2)
# By using country, it can return a more accurate location as the same postcode may exist in multiple countries
# Coordinates of the country will be returned if postcode is invalid in that country
def lookup_coord(postcode, country):

    payload = {"address": postcode, "components": "country:" + country}

    return get_location_from_api(payload)


# Using the Google Map API, lookup country using postcode
def lookup_country(postcode):

    payload = {"components": "postal_code:" + postcode}

    return get_location_from_api(payload)


# Takes in payload as argument and use it when calling API
# Catches any error and return an error message as first element in the tuple
# Coordinates of GB is used if country is not specified, otherwise default to original country and 0 for coordinates
# Return format is:
#     error, country, town, latitude, longitude
def get_location_from_api(payload):
    # default location is UK and the coordinates of UK
    default_country = "GB"
    default_town = 0
    default_lat = 55.378051
    default_lng = -3.435973

    if "country" in payload.get("components"):
        default_country = payload.get("components", "GB").rpartition(":")[2]
        default_town = default_lat = default_lng = 0

    # Catch error when there is a problem connecting to external API
    try:
        res = requests.get(
            "https://maps.googleapis.com/maps/api/geocode/json", params=payload
        )

        # Make sure status_code is 200 before deserialising json
        if not res.status_code == requests.codes.ok:
            raise RequestException(res.reason)

        data = res.json()

        status = data.get("status", "No status")
        results = data.get("results", [])

        if not (status == "OK" and len(results) > 0):
            raise ApiException(status)

        country, town, lat, lng = extract_location_data(results)

        return (
            None,
            country or default_country,
            town or default_town,
            lat or default_lat,
            lng or default_lng,
        )

    except requests.exceptions.RequestException:
        error = "Connection error"

    except RequestException as e:
        error = "Request error: %s" % e

    except ValueError as e:
        error = "Value error: %s" % e

    except ApiException as e:
        error = "API error: %s" % e

    return error, default_country, default_town, default_lat, default_lng



from django.contrib import messages
from django.contrib.auth.views import LoginView
from django.core.urlresolvers import reverse_lazy
from django.utils.html import escape

from portal.forms.play import StudentLoginForm


class StudentLoginView(LoginView):
    template_name = "portal/login/student.html"
    form_class = StudentLoginForm
    success_url = reverse_lazy("student_details")
    redirect_authenticated_user = reverse_lazy("student_details")

    def _add_logged_in_as_message(self, request):
        student = request.user.userprofile.student
        student_class = student.class_field
        student_school = student_class.teacher.school

        messages.info(
            request,
            (
                "You are logged in as a member of class: <strong>"
                + escape(student_class.name)
                + "</strong>, in school or club: <strong>"
                + escape(student_school.name)
                + "</strong>."
            ),
            extra_tags="safe",
        )

    def get_success_url(self):
        redirect_url = self.get_redirect_url()

        if redirect_url:
            return redirect_url

        self._add_logged_in_as_message(self.request)

        return self.success_url



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2018, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from __future__ import absolute_import

from builtins import object
import time

from selenium.common.exceptions import TimeoutException
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait

FADE_TIME = 0.16


class BasePage(object):
    browser = None

    DEFAULT_WAIT_SECONDS = 5

    def __init__(self, browser):
        self.browser = browser

    def wait_for_element_by_id(self, id, wait_seconds=DEFAULT_WAIT_SECONDS):
        self.wait_for_presence((By.ID, id), wait_seconds)

    def wait_for_element_by_css(self, css, wait_seconds=DEFAULT_WAIT_SECONDS):
        self.wait_for_presence((By.CSS_SELECTOR, css), wait_seconds)

    def wait_for_element_to_be_clickable(
        self, locator, wait_seconds=DEFAULT_WAIT_SECONDS
    ):
        self.wait(EC.element_to_be_clickable(locator), wait_seconds)

    def wait_for_element_to_be_invisible(
        self, locator, wait_seconds=DEFAULT_WAIT_SECONDS
    ):
        self.wait(EC.invisibility_of_element_located(locator), wait_seconds)

    def wait_for_presence(self, locator, wait_seconds=DEFAULT_WAIT_SECONDS):
        self.wait(EC.presence_of_element_located(locator), wait_seconds)

    def wait_for_absence(self, locator, wait_seconds=DEFAULT_WAIT_SECONDS):
        self.wait_until_not(EC.presence_of_element_located(locator), wait_seconds)

    def wait(self, method, wait_seconds=DEFAULT_WAIT_SECONDS):
        WebDriverWait(self.browser, wait_seconds).until(method)

    def wait_until_not(self, method, wait_seconds=DEFAULT_WAIT_SECONDS):
        WebDriverWait(self.browser, wait_seconds).until_not(method)

    def element_exists(self, locator):
        try:
            self.wait_for_presence(locator)
            return True
        except TimeoutException:
            return False

    def element_does_not_exist(self, locator):
        try:
            self.wait_for_absence(locator)
            return True
        except TimeoutException:
            return False

    def element_does_not_exist_by_id(self, name):
        return self.element_does_not_exist((By.ID, name))

    def element_exists_by_id(self, name):
        return self.element_exists((By.ID, name))

    def element_does_not_exist_by_link_text(self, name):
        return self.element_does_not_exist((By.LINK_TEXT, name))

    def element_exists_by_css(self, name):
        return self.element_exists((By.CSS_SELECTOR, name))

    def on_correct_page(self, pageName):
        return self.element_exists_by_id(pageName)

    def go_to_resources_page(self):
        self.browser.find_element_by_id("resources_button").click()
        return ResourcesPage(self.browser)

    def go_to_aimmo_home_page(self):
        self.browser.find_element_by_id("aimmo_home_button").click()
        return aimmo_home_page.AimmoHomePage(self.browser)

    def is_on_admin_login_page(self):
        return self.on_correct_page("administration_login")

    def is_on_admin_data_page(self):
        return self.on_correct_page("admin_data")

    def is_on_admin_map_page(self):
        return self.on_correct_page("admin_map")

    def is_on_403_forbidden(self):
        return self.on_correct_page("403_forbidden")

    def was_form_invalid(self, formID, error):
        errors = (
            self.browser.find_element_by_id(formID)
            .find_element_by_class_name("errorlist")
            .text
        )
        return error in errors

    def is_dialog_showing(self):
        time.sleep(FADE_TIME)
        return self.browser.find_element_by_id("popup").is_displayed()

    def confirm_dialog(self):
        self.browser.find_element_by_id("confirm_button").click()
        return self

    def cancel_dialog(self):
        self.browser.find_element_by_id("cancel_button").click()
        time.sleep(FADE_TIME)
        return self


from .resources_page import ResourcesPage
from . import aimmo_home_page



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2018, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from __future__ import absolute_import
import time

from selenium.webdriver.support.ui import Select

from . import email_verification_needed_page
from .base_page import BasePage


class SignupPage(BasePage):
    def __init__(self, browser):
        super(SignupPage, self).__init__(browser)

        assert self.on_correct_page("signup_page")

    def signup(
        self,
        title,
        first_name,
        last_name,
        email,
        password,
        confirm_password,
        success=True,
        newsletter=False,
    ):
        Select(
            self.browser.find_element_by_id("id_teacher_signup-teacher_title")
        ).select_by_value(title)
        self.browser.find_element_by_id(
            "id_teacher_signup-teacher_first_name"
        ).send_keys(first_name)
        self.browser.find_element_by_id(
            "id_teacher_signup-teacher_last_name"
        ).send_keys(last_name)
        self.browser.find_element_by_id("id_teacher_signup-teacher_email").send_keys(
            email
        )
        self.browser.find_element_by_id("id_teacher_signup-teacher_password").send_keys(
            password
        )
        self.browser.find_element_by_id(
            "id_teacher_signup-teacher_confirm_password"
        ).send_keys(confirm_password)

        if newsletter:
            self.browser.find_element_by_id(
                "id_teacher_signup-newsletter_ticked"
            ).click()

        self.browser.find_element_by_name("teacher_signup").click()

        if success:
            return email_verification_needed_page.EmailVerificationNeededPage(
                self.browser
            )
        else:
            return self

    def independent_student_signup(
        self,
        name,
        username,
        email_address,
        password,
        confirm_password,
        success=True,
        newsletter=False,
    ):
        self.browser.find_element_by_id("id_independent_student_signup-name").send_keys(
            name
        )
        self.browser.find_element_by_id(
            "id_independent_student_signup-username"
        ).send_keys(username)
        self.browser.find_element_by_id(
            "id_independent_student_signup-email"
        ).send_keys(email_address)
        self.browser.find_element_by_id(
            "id_independent_student_signup-password"
        ).send_keys(password)
        self.browser.find_element_by_id(
            "id_independent_student_signup-confirm_password"
        ).send_keys(confirm_password)

        if newsletter:
            self.browser.find_element_by_id(
                "id_teacher_signup-newsletter_ticked"
            ).click()

        self.browser.find_element_by_name("independent_student_signup").click()
        if success:
            from .email_verification_needed_page import EmailVerificationNeededPage

            return EmailVerificationNeededPage(self.browser)
        else:
            return self

    def has_independent_student_signup_failed(self, error):
        time.sleep(1.5)
        errors = (
            self.browser.find_element_by_id("form-signup-independent-student")
            .find_element_by_class_name("errorlist")
            .text
        )
        return error in errors

    def has_teacher_signup_failed(self, error):
        time.sleep(1.5)
        errors = (
            self.browser.find_element_by_id("form-reg-teacher")
            .find_element_by_class_name("errorlist")
            .text
        )
        return error in errors



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program

import datetime

from common.models import Teacher, Student
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.http import HttpResponse
from django.utils import timezone
from rest_framework import serializers, permissions, generics, status
from rest_framework.authentication import SessionAuthentication
from rest_framework.decorators import api_view
from rest_framework.response import Response
from rest_framework.reverse import reverse_lazy

THREE_YEARS_IN_DAYS = 1095


@api_view(("GET",))
@login_required(login_url=reverse_lazy("administration_login"))
def registered_users(request, year, month, day):
    try:
        nbr_reg = User.objects.filter(
            date_joined__startswith=datetime.date(int(year), int(month), int(day))
        ).count()
        return Response(nbr_reg)
    except ValueError:
        return HttpResponse(status=404)


@api_view(("GET",))
@login_required(login_url=reverse_lazy("administration_login"))
def last_connected_since(request, year, month, day):
    try:
        nbr_active_users = User.objects.filter(
            last_login__gte=datetime.date(int(year), int(month), int(day))
        ).count()
        return Response(nbr_active_users)
    except ValueError:
        return HttpResponse(status=404)


@api_view(("GET",))
@login_required(login_url=reverse_lazy("administration_login"))
def number_users_per_country(request, country):
    try:
        nbr_reg = (
            Teacher.objects.filter(school__country__exact=country).count()
            + Student.objects.filter(
                class_field__teacher__school__country__exact=country
            ).count()
        )
        return Response(nbr_reg)
    except ValueError:
        return HttpResponse(status=404)


class InactiveUserSerializer(serializers.Serializer):
    """The user information we show in the InactiveUsersViewSet."""

    username = serializers.CharField()
    first_name = serializers.CharField()
    last_name = serializers.CharField()
    email = serializers.CharField()
    date_joined = serializers.DateTimeField()
    last_login = serializers.DateTimeField()


class IsAdminOrGoogleAppEngine(permissions.IsAdminUser):
    """Checks whether the request is from a Google App Engine cron job."""

    def has_permission(self, request, view):
        return request.META.get("HTTP_X_APPENGINE_CRON") is not None or super(
            IsAdminOrGoogleAppEngine, self
        ).has_permission(request, view)


class InactiveUsersView(generics.ListAPIView):
    """
    This API view endpoint allows us to see our inactive users.

    An inactive user is one that hasn't logged in for three years.
    If the user has never logged in, we look at the date they registered with us instead.
    """

    queryset = User.objects.filter(
        last_login__lte=timezone.now() - timezone.timedelta(days=THREE_YEARS_IN_DAYS)
    ) | User.objects.filter(
        last_login__isnull=True,
        date_joined__lte=timezone.now() - timezone.timedelta(days=THREE_YEARS_IN_DAYS),
    )
    authentication_classes = (SessionAuthentication,)
    serializer_class = InactiveUserSerializer
    permission_classes = (IsAdminOrGoogleAppEngine,)

    def delete(self, request):
        """Delete all inactive users."""
        inactive_users = self.get_queryset()
        inactive_users.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from builtins import str
from datetime import timedelta
from time import sleep

from common.models import Teacher, School, Class, Student
from django.contrib import messages as messages
from django.contrib.auth.decorators import permission_required, login_required
from django.contrib.auth.models import User
from django.contrib.auth.views import LoginView
from django.db.models import Avg, Count, Q
from django.shortcuts import render
from django.utils import timezone
from django_otp import device_classes
from rest_framework.reverse import reverse_lazy

from portal import app_settings
from portal.forms.admin_login import AdminLoginForm
from portal.helpers.location import lookup_coord

block_limit = 5


def is_post_request(request, response):
    return request.method == "POST"


class AdminLoginView(LoginView):
    form_class = AdminLoginForm
    template_name = "registration/login.html"
    extra_context = {"settings": app_settings}
    authentication_form = None

    def get_form_kwargs(self):
        kwargs = super(LoginView, self).get_form_kwargs()
        return kwargs

    def get_form(self, form_class=None):
        user = self.request.user
        return self.form_class(user, **self.get_form_kwargs())


@login_required(login_url=reverse_lazy("administration_login"))
@permission_required("common.view_aggregated_data", raise_exception=True)
def aggregated_data(request):

    tables = []

    table_head = ["Data description", "Value", "More info"]
    table_data = []

    """
    Overall statistics
    """

    teacher_count = Teacher.objects.count()
    student_count = Student.objects.count()
    new_profiles_count = User.objects.filter(
        date_joined__gte=timezone.now() - timedelta(days=7)
    ).count()

    table_data.append(
        [
            "Number of users",
            teacher_count + student_count,
            "Number of teachers + Number of students",
        ]
    )

    table_data.append(
        [
            "Number of new users (past week)",
            new_profiles_count,
            "Number of user profiles",
        ]
    )

    tables.append(
        {
            "title": "Overall Statistics",
            "description": "CFL site overall statistics",
            "header": table_head,
            "data": table_data,
        }
    )

    """
    School statistics
    """
    table_data = []
    table_data.append(["Number of schools signed up", School.objects.count(), ""])
    num_of_teachers_per_school = School.objects.annotate(
        num_teachers=Count("teacher_school")
    )
    stats_teachers_per_school = num_of_teachers_per_school.aggregate(
        Avg("num_teachers")
    )

    table_data.append(
        [
            "Average number of teachers per school",
            stats_teachers_per_school["num_teachers__avg"],
            "",
        ]
    )

    tables.append(
        {
            "title": "Schools or Clubs",
            "description": "",
            "header": table_head,
            "data": table_data,
        }
    )

    """
    Teacher statistics
    """
    table_data = []
    table_data.append(["Number of teachers signed up", teacher_count, ""])

    table_data.append(
        [
            "Number of teachers not in a school",
            Teacher.objects.filter(school=None).count(),
            "",
        ]
    )

    table_data.append(
        [
            "Number of teachers with request pending to join a school",
            Teacher.objects.exclude(pending_join_request=None).count(),
            "",
        ]
    )

    table_data.append(
        [
            "Number of teachers with unverified email address",
            Teacher.objects.exclude(
                new_user__email_verifications__verified=True
            ).count(),
            "",
        ]
    )

    otp_model_names = [model._meta.model_name for model in device_classes()]
    otp_query = Q()
    for model_name in otp_model_names:
        otp_query = otp_query | Q(**{"new_user__%s__name" % model_name: "default"})
    two_factor_teachers = Teacher.objects.filter(otp_query).distinct().count()
    table_data.append(["Number of teachers setup with 2FA", two_factor_teachers, ""])
    num_of_classes_per_teacher = Teacher.objects.annotate(
        num_classes=Count("class_teacher")
    )
    stats_classes_per_teacher = num_of_classes_per_teacher.aggregate(Avg("num_classes"))
    num_of_classes_per_active_teacher = num_of_classes_per_teacher.exclude(school=None)
    stats_classes_per_active_teacher = num_of_classes_per_active_teacher.aggregate(
        Avg("num_classes")
    )

    table_data.append(
        [
            "Average number of classes per teacher",
            stats_classes_per_teacher["num_classes__avg"],
            "",
        ]
    )

    table_data.append(
        [
            "Average number of classes per active teacher",
            stats_classes_per_active_teacher["num_classes__avg"],
            "Excludes teachers without a school",
        ]
    )

    table_data.append(
        [
            "Number of of teachers with no classes",
            num_of_classes_per_teacher.filter(num_classes=0).count(),
            "",
        ]
    )

    table_data.append(
        [
            "Number of of active teachers with no classes",
            num_of_classes_per_active_teacher.filter(num_classes=0).count(),
            "Excludes teachers without a school",
        ]
    )

    tables.append(
        {
            "title": "Teachers",
            "description": "",
            "header": table_head,
            "data": table_data,
        }
    )

    """
    Class statistics
    """
    table_data = []
    table_data.append(["Number of classes", Class.objects.count(), ""])

    num_students_per_class = Class.objects.annotate(num_students=Count("students"))
    stats_students_per_class = num_students_per_class.aggregate(Avg("num_students"))
    stats_students_per_active_class = num_students_per_class.exclude(
        num_students=0
    ).aggregate(Avg("num_students"))

    table_data.append(
        [
            "Average number of students per class",
            stats_students_per_class["num_students__avg"],
            "",
        ]
    )

    table_data.append(
        [
            "Average number of students per active class",
            stats_students_per_active_class["num_students__avg"],
            "Excludes classes which are empty",
        ]
    )

    tables.append(
        {
            "title": "Classes",
            "description": "",
            "header": table_head,
            "data": table_data,
        }
    )

    """
    Student statistics
    """
    table_data = []
    table_data.append(["Number of students", student_count, ""])

    independent_students = Student.objects.filter(class_field=None)

    table_data.append(
        ["Number of independent students", independent_students.count(), ""]
    )

    table_data.append(
        [
            "Number of independent students with unverified email address",
            Student.objects.exclude(
                new_user__email_verifications__verified=True
            ).count(),
            "",
        ]
    )

    table_data.append(
        [
            "Number of school students",
            Student.objects.exclude(class_field=None).count(),
            "",
        ]
    )

    tables.append(
        {
            "title": "Students",
            "description": "",
            "header": table_head,
            "data": table_data,
        }
    )

    """
    Rapid Router Student Progress statistics
    """
    table_data = []

    students_with_attempts = Student.objects.annotate(
        num_attempts=Count("attempts")
    ).exclude(num_attempts=0)
    table_data.append(
        ["Number of students who have started RR", students_with_attempts.count(), ""]
    )

    school_students_with_attempts = students_with_attempts.exclude(class_field=None)
    table_data.append(
        [
            "Number of school students who have started RR",
            school_students_with_attempts.count(),
            "",
        ]
    )

    independent_students_with_attempts = students_with_attempts.filter(class_field=None)
    table_data.append(
        [
            "Number of independent students who have started RR",
            independent_students_with_attempts.count(),
            "",
        ]
    )

    tables.append(
        {
            "title": "Rapid Router Student Progress",
            "description": "",
            "header": table_head,
            "data": table_data,
        }
    )

    return render(request, "portal/admin/aggregated_data.html", {"tables": tables})


def fill_in_missing_school_locations(request):
    schools = School.objects.filter(latitude="0", longitude="0")

    requests = 0
    failures = []
    town0 = 0

    for school in schools:
        requests += 1
        sleep(0.2)  # so we execute a bit less than 5/sec

        (
            error,
            school.country,
            school.town,
            school.latitude,
            school.longitude,
        ) = lookup_coord(school.postcode, school.country.code)

        if error is None:
            school.save()

        if error is not None:
            failures += [(school.id, school.postcode, error)]

        if school.town == "0":
            town0 += 1

    messages.info(request, "Made %d requests" % requests)
    messages.info(request, "There were %d errors: %s" % (len(failures), str(failures)))
    messages.info(request, "%d school have no town" % town0)


@login_required(login_url=reverse_lazy("administration_login"))
@permission_required("common.view_map_data", raise_exception=True)
def schools_map(request):
    fill_in_missing_school_locations(request)

    return render(request, "portal/admin/map.html", {"schools": School.objects.all()})



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from __future__ import absolute_import

import time

from common.tests.utils.classes import create_class_directly
from common.tests.utils.student import (
    create_many_school_students,
    create_school_student,
    create_school_student_directly,
)
from common.tests.utils.teacher import signup_teacher_directly
from portal.tests.pageObjects.portal.home_page import HomePage

from .base_test import BaseTest
from .utils.organisation import (
    create_organisation_directly,
    join_teacher_to_organisation,
)


class TestTeacherStudent(BaseTest):
    def test_create(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, _ = create_class_directly(email)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login_no_students(email, password)
        )

        page, student_name = create_school_student(page)
        assert page.student_exists(student_name)

        assert page.__class__.__name__ == "OnboardingStudentListPage"

    def test_create_valid_name_dash(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, _ = create_class_directly(email)

        student_name = "Florian-Gilbert"

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login_no_students(email, password)
        )

        page = page.type_student_name(student_name).create_students()

        assert page.student_exists(student_name)

        assert page.__class__.__name__ == "OnboardingStudentListPage"

    def test_create_valid_name_underscore(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, _ = create_class_directly(email)

        student_name = "Florian_Gilbert"

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login_no_students(email, password)
        )

        page = page.type_student_name(student_name).create_students()

        assert page.student_exists(student_name)

        assert page.__class__.__name__ == "OnboardingStudentListPage"

    def test_create_invalid_name(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, _ = create_class_directly(email)

        student_name = "Florian!"

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login_no_students(email, password)
        )

        page = page.type_student_name(student_name).create_students_failure()

        assert page.adding_students_failed()
        assert page.was_form_invalid(
            "form-create-students",
            "Names may only contain letters, numbers, dashes, underscores, and spaces.",
        )

    def test_create_multiple(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, _ = create_class_directly(email)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login_no_students(email, password)
        )

        page, student_names = create_many_school_students(page, 12)

        for student_name in student_names:
            assert page.student_exists(student_name)

    def test_create_duplicate(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, _ = create_class_directly(email)

        student_name = "bob"

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login_no_students(email, password)
        )

        page = (
            page.type_student_name(student_name)
            .type_student_name(student_name)
            .create_students_failure()
        )
        assert page.adding_students_failed()
        assert page.duplicate_students(student_name)

    def test_add_to_existing_class(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        create_school_student_directly(access_code)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login(email, password)
            .go_to_class_page()
        )

        page, new_student_name = create_school_student(page)
        assert page.student_exists(new_student_name)

        page = page.go_back_to_class()

        assert page.student_exists(new_student_name)

    def test_update_student_name(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        name, password, _ = create_school_student_directly(access_code)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login(email, password)
            .go_to_class_page()
            .go_to_edit_student_page()
        )

        assert page.is_student_name(name)

        new_student_name = "new name"

        page = page.type_student_name(new_student_name)
        page = page.click_update_button()

        assert page.is_student_name(new_student_name)

    def test_update_student_valid_name_dash(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        name, password, _ = create_school_student_directly(access_code)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login(email, password)
            .go_to_class_page()
            .go_to_edit_student_page()
        )

        assert page.is_student_name(name)

        new_student_name = "new-name"

        page = page.type_student_name(new_student_name)
        page = page.click_update_button()

        assert page.is_student_name(new_student_name)

    def test_update_student_valid_name_underscore(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        name, password, _ = create_school_student_directly(access_code)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login(email, password)
            .go_to_class_page()
            .go_to_edit_student_page()
        )

        assert page.is_student_name(name)

        new_student_name = "new_name"

        page = page.type_student_name(new_student_name)
        page = page.click_update_button()

        assert page.is_student_name(new_student_name)

    def test_update_student_invalid_name(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        name, password, _ = create_school_student_directly(access_code)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login(email, password)
            .go_to_class_page()
            .go_to_edit_student_page()
        )

        assert page.is_student_name(name)

        new_student_name = "new name!"

        page = page.type_student_name(new_student_name)
        page = page.click_update_button()

        assert page.is_student_name(name)
        assert page.was_form_invalid(
            "form-edit-student",
            "Names may only contain letters, numbers, dashes, underscores, and spaces.",
        )

    def test_update_student_password(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        name, password, _ = create_school_student_directly(access_code)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login(email, password)
            .go_to_class_page()
            .go_to_edit_student_page()
        )

        assert page.is_student_name(name)

        new_student_password = "New_password1"

        page = page.click_set_password_form_button().type_student_password(
            new_student_password
        )
        time.sleep(1)
        page = page.click_set_password_button()

        assert page.is_student_password(new_student_password)

    def test_generate_random_student_password(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        name, password, _ = create_school_student_directly(access_code)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login(email, password)
            .go_to_class_page()
            .go_to_edit_student_page()
        )

        assert page.is_student_name(name)

        page = page.click_generate_password_button()

        assert page.__class__.__name__ == "EditStudentPasswordPage"

    def test_delete(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        student_name, _, _ = create_school_student_directly(access_code)

        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium).go_to_teacher_login_page().login(email, password)
        page = page.go_to_class_page()
        assert page.student_exists(student_name)

        page = page.toggle_select_student().delete_students()
        assert page.is_dialog_showing()
        page = page.confirm_delete_student_dialog()

        assert not page.student_exists(student_name)

    def test_reset_passwords(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        student_name, _, _ = create_school_student_directly(access_code)

        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium).go_to_teacher_login_page().login(email, password)
        page = page.go_to_class_page()
        assert page.student_exists(student_name)

        page = page.toggle_select_student().reset_passwords()
        assert page.is_dialog_showing()
        page = page.confirm_reset_student_dialog()

        assert page.student_exists(student_name)
        assert page.__class__.__name__ == "OnboardingStudentListPage"

    def test_move_cancel(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        _, _, _ = create_school_student_directly(access_code)

        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium).go_to_teacher_login_page().login(email, password)
        page = page.go_to_class_page()

        page = page.move_students_none_selected()
        assert page.__class__.__name__ == "TeachClassPage"

        page = page.toggle_select_student().move_students()
        assert page.__class__.__name__ == "TeachMoveStudentsPage"

        page = page.cancel()
        assert page.__class__.__name__ == "TeachClassPage"

    def test_move_cancel_disambiguate(self):
        old_teacher_email, password_1 = signup_teacher_directly()
        email_2, _ = signup_teacher_directly()
        org_name, postcode = create_organisation_directly(old_teacher_email)
        join_teacher_to_organisation(email_2, org_name, postcode)
        _, _, access_code_1 = create_class_directly(old_teacher_email)
        _, _, _ = create_class_directly(email_2)
        student_name, _, _ = create_school_student_directly(access_code_1)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login(old_teacher_email, password_1)
        )
        page = page.go_to_class_page()
        assert page.has_students()
        assert page.student_exists(student_name)

        page = page.toggle_select_student()
        page = page.move_students().select_class_by_index(0).move().cancel()
        assert page.has_students()
        assert page.student_exists(student_name)

    def test_move(self):
        email_1, password_1 = signup_teacher_directly()
        email_2, password_2 = signup_teacher_directly()
        org_name, postcode = create_organisation_directly(email_1)
        join_teacher_to_organisation(email_2, org_name, postcode)
        _, _, access_code_1 = create_class_directly(email_1)
        _, _, _ = create_class_directly(email_2)
        student_name_1, _, _ = create_school_student_directly(access_code_1)
        student_name_2, _, _ = create_school_student_directly(access_code_1)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login(email_1, password_1)
        )
        page = page.go_to_class_page()
        assert page.student_exists(student_name_1)
        assert page.student_exists(student_name_2)

        page = page.toggle_select_student()
        page = page.move_students().select_class_by_index(0).move().move()
        assert not page.student_exists(student_name_1)

        page = page.go_to_dashboard()
        page = (
            page.go_to_top()
            .logout()
            .go_to_teacher_login_page()
            .login(email_2, password_2)
        )
        page = page.go_to_class_page()
        assert page.student_exists(student_name_1)

    def test_dismiss(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        student_name_1, _, _ = create_school_student_directly(access_code)
        _, _, _ = create_school_student_directly(access_code)

        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium).go_to_teacher_login_page().login(email, password)
        page = page.go_to_class_page()
        assert page.student_exists(student_name_1)

        page = page.toggle_select_student().dismiss_students()
        assert page.__class__.__name__ == "TeachDismissStudentsPage"
        page = page.cancel()
        assert page.__class__.__name__ == "TeachClassPage"

        page = (
            page.toggle_select_student()
            .dismiss_students()
            .enter_email("student_email@gmail.com")
            .dismiss()
        )
        assert not page.student_exists(student_name_1)



from common.tests.base_test_migration import MigrationTestCase


class TestMigrationUseCommonModels(MigrationTestCase):

    start_migration = "0057_delete_frontpagenews"
    dest_migration = "0058_move_to_common_models"

    def test_models_moved_to_common(self):
        model_names = [
            model._meta.db_table for model in self.django_application.get_models()
        ]

        moved_models = ["userprofile", "school", "teacher", "class", "student"]

        for moved_model in moved_models:
            assert f"portal_{moved_model}" not in model_names
            assert f"common_{moved_model}" in model_names


class TestMigrationMoveEmailVerificationToCommon(MigrationTestCase):

    start_migration = "0058_move_to_common_models"
    dest_migration = "0059_move_email_verifications_to_common"

    def test_emailverification_moved_to_common(self):
        model_names = [
            model._meta.db_table for model in self.django_application.get_models()
        ]

        assert f"portal_emailverification" not in model_names
        assert f"common_emailverification" in model_names



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2018, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from __future__ import absolute_import

import time

from selenium.webdriver.support.ui import Select

from .add_independent_student_to_class_page import AddIndependentStudentToClassPage
from .class_page import TeachClassPage
from .move_classes_page import TeachMoveClassesPage
from .teach_base_page import TeachBasePage


class TeachDashboardPage(TeachBasePage):
    def __init__(self, browser):
        super(TeachDashboardPage, self).__init__(browser)

        assert self.on_correct_page("teach_dashboard_page")

    def go_to_class_page(self):
        self.browser.find_element_by_id("class_button").click()
        return TeachClassPage(self.browser)

    def go_to_top(self):
        self.browser.find_element_by_id("back_to_top_button").click()
        time.sleep(3)
        return self

    def check_organisation_details(self, details):
        correct = True

        first_field = list(details.items())[0][0]
        self.wait_for_element_by_id("id_" + first_field)

        for field, value in list(details.items()):
            correct &= (
                self.browser.find_element_by_id("id_" + field).get_attribute("value")
                == value
            )

        return correct

    def change_organisation_details(self, details):
        for field, value in list(details.items()):
            self.browser.find_element_by_id("id_" + field).clear()
            self.browser.find_element_by_id("id_" + field).send_keys(value)

        self.browser.find_element_by_id("update_details_button").click()
        return self

    def has_edit_failed(self):
        self.wait_for_element_by_id("edit_form")
        errorlist = (
            self.browser.find_element_by_id("edit_form")
            .find_element_by_class_name("errorlist")
            .text
        )
        error = (
            "There is already a school or club registered with that name and postcode"
        )
        return error in errorlist

    def create_class(self, name, classmate_progress):
        self.browser.find_element_by_id("id_class_name").send_keys(name)
        Select(
            self.browser.find_element_by_id("id_classmate_progress")
        ).select_by_value(classmate_progress)

        self.browser.find_element_by_id("create_class_button").click()

        return self

    def change_teacher_details(self, details):
        self._change_details(details)

        return self

    def change_email(self, first_name, last_name, new_email, password):
        self._change_details(
            {
                "first_name": first_name,
                "last_name": last_name,
                "email": new_email,
                "current_password": password,
            }
        )

        from portal.tests.pageObjects.portal.email_verification_needed_page import (
            EmailVerificationNeededPage,
        )

        return EmailVerificationNeededPage(self.browser)

    def _change_details(self, details):
        if "title" in details:
            Select(self.browser.find_element_by_id("id_title")).select_by_value(
                details["title"]
            )
            del details["title"]
        for field, value in list(details.items()):
            self.browser.find_element_by_id("id_" + field).clear()
            self.browser.find_element_by_id("id_" + field).send_keys(value)
        self.browser.find_element_by_id("update_button").click()

    def check_account_details(self, details):
        correct = True

        if "title" in details:
            correct &= (
                Select(
                    self.browser.find_element_by_id("id_title")
                ).first_selected_option.text
                == details["title"]
            )
            del details["title"]

        for field, value in list(details.items()):
            correct &= (
                self.browser.find_element_by_id("id_" + field).get_attribute("value")
                == value
            )

        return correct

    def accept_join_request(self):
        self.browser.find_element_by_id("requests_button").click()
        time.sleep(3)
        self.browser.find_element_by_id("allow_button").click()
        return self

    def deny_join_request(self):
        self.browser.find_element_by_id("requests_button").click()
        time.sleep(3)
        self.browser.find_element_by_id("deny_button").click()
        return self

    def accept_independent_join_request(self):
        self.browser.find_element_by_id("allow_independent_button").click()
        return AddIndependentStudentToClassPage(self.browser)

    def deny_independent_join_request(self):
        self.browser.find_element_by_id("deny_independent_button").click()
        return self

    def has_join_request(self, email):
        return self.element_exists_by_id("request_table") and (
            email in self.browser.find_element_by_id("request_table").text
        )

    def has_no_join_requests(self):
        return self.element_does_not_exist_by_id("request_table")

    def has_independent_join_request(self, email):
        return self.element_exists_by_id("independent_request_table") and (
            email in self.browser.find_element_by_id("independent_request_table").text
        )

    def has_no_independent_join_requests(self):
        return self.element_does_not_exist_by_id("independent_request_table")

    def is_teacher_in_school(self, name):
        return name in self.browser.find_element_by_id("teachers_table").text

    def is_not_teacher_in_school(self, name):
        return name not in self.browser.find_element_by_id("teachers_table").text

    def is_teacher_admin(self):
        return (
            "Make non-admin" in self.browser.find_element_by_id("teachers_table").text
        )

    def have_classes(self):
        return self.element_exists_by_id("classes-table")

    def does_not_have_classes(self):
        return self.element_does_not_exist_by_id("classes-table")

    def does_class_exist(self, name, access_code):
        return (
            self.have_classes()
            and (name in self.browser.find_element_by_id("classes-table").text)
            and (access_code in self.browser.find_element_by_id("classes-table").text)
        )

    def is_teacher_non_admin(self):
        return "Make admin" in self.browser.find_element_by_id("teachers_table").text

    def click_kick_button(self):
        self.browser.find_element_by_id("kick_button").click()

        return self

    def click_make_admin_button(self):
        self.browser.find_element_by_id("make_admin_button").click()

        return self

    def click_make_non_admin_button(self):
        self.browser.find_element_by_id("make_non_admin_button").click()

        return self

    def leave_organisation_with_students(self):
        self._click_leave_button()

        return TeachMoveClassesPage(self.browser)

    def _click_leave_button(self):
        self.browser.find_element_by_id("leave_organisation_button").click()

    def confirm_kick_with_students_dialog(self):
        self.confirm_dialog()

        return TeachMoveClassesPage(self.browser)



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2018, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from __future__ import division
from builtins import str
import game.messages as messages
from django.template import RequestContext
from django.utils.safestring import mark_safe

from game.models import Episode
from django.core.cache import cache
from game import app_settings
from common.permissions import logged_in_as_teacher
from django.contrib.auth.decorators import login_required, user_passes_test
from django.core.urlresolvers import reverse_lazy
from django.shortcuts import render, get_object_or_404


def fetch_episode_data_from_database(early_access):
    episode_data = []
    episode = Episode.objects.get(pk=1)
    while episode is not None:
        if episode.in_development and not early_access:
            break

        levels, minName, maxName = min_max_levels(episode.levels)

        e = {
            "id": episode.id,
            "name": episode.name,
            "levels": levels,
            "first_level": minName,
            "last_level": maxName,
            "random_levels_enabled": episode.r_random_levels_enabled,
        }

        episode_data.append(e)
        episode = episode.next_episode
    return episode_data


def min_max_levels(episode_levels):
    levels = []
    minName = 1000
    maxName = 0

    for level in episode_levels:
        level_name = int(level.name)
        if level_name > maxName:
            maxName = level_name
        if level_name < minName:
            minName = level_name

        levels.append(
            {"id": level.id, "name": level_name, "title": get_level_title(level_name)}
        )

    return levels, minName, maxName


def fetch_episode_data(early_access):
    key = "episode_data"
    if early_access:
        key = "episode_data_early_access"
    data = cache.get(key)
    if data is None:
        data = fetch_episode_data_from_database(early_access)
        cache.set(key, data)
    return data


def get_level_title(i):
    title = "title_level" + str(i)
    try:
        titleCall = getattr(messages, title)
        return mark_safe(titleCall())
    except AttributeError:
        return ""


@login_required(login_url=reverse_lazy("teach"))
@user_passes_test(logged_in_as_teacher, login_url=reverse_lazy("teach"))
def levels(request):
    """ Loads a page with all levels listed.

    **Context**

    ``RequestContext``
    ``episodes``

    **Template:**

    :template:`game/level_selection.html`
    """

    episode_data = fetch_episode_data(app_settings.EARLY_ACCESS_FUNCTION(request))

    return render(
        request,
        "portal/teach/teacher_level_solutions.html",
        {"episodeData": episode_data},
    )



# coding=utf-8
HOME_LEARNING_BANNER = {
    "title": "#KeepKidsCoding",
    "subtitle": "Teach your child to code at home — our Rapid Router game supports the "
                "National Curriculum Computing strand, is easy to teach and free to "
                "use – forever.",
    "text": "",
    "image_class": "banner--picture--educate",
}



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from common.models import Teacher, Student
from django.contrib import messages as messages
from django.contrib.auth import logout
from django.core.urlresolvers import reverse, reverse_lazy
from django.http import HttpResponseRedirect, HttpResponse
from django.shortcuts import render
from django.views.decorators.cache import cache_control
from django.views.decorators.csrf import csrf_exempt

from deploy import captcha
from common import email_messages
from portal.forms.newsletter_form import NewsletterForm
from portal.forms.play import IndependentStudentSignupForm
from portal.forms.teach import TeacherSignupForm
from portal.helpers.captcha import remove_captcha_from_forms
from common.helpers.emails import (
    send_verification_email,
    send_email,
    NOTIFICATION_EMAIL,
    add_to_dotmailer,
)
from common.permissions import logged_in_as_student, logged_in_as_teacher
from portal.strings.home_learning import HOME_LEARNING_BANNER


def teach_email_labeller(request):
    if request.method == "POST" and "teacher_login" in request.POST:
        return request.POST["login-teacher_email"]

    return ""


def play_name_labeller(request):
    if request.method == "POST":
        if "school_login" in request.POST:
            return request.POST["login-name"] + ":" + request.POST["login-access_code"]

        if "independent_student_login" in request.POST:
            return request.POST["independent_student-username"]

    return ""


def register_view(request):
    if request.user.is_authenticated():
        return redirect_user_to_dashboard(request)
    else:
        return render_signup_form(request)


def redirect_user_to_dashboard(request):
    if logged_in_as_student(request.user):
        return HttpResponseRedirect(reverse_lazy("student_details"))
    elif logged_in_as_teacher(request.user):
        return HttpResponseRedirect(reverse_lazy("dashboard"))
    else:
        return HttpResponseRedirect(reverse_lazy("home"))


def logout_view(request):
    logout(request)
    return HttpResponseRedirect(reverse_lazy("home"))


def render_signup_form(request):
    invalid_form = False

    teacher_signup_form = TeacherSignupForm(prefix="teacher_signup")
    independent_student_signup_form = IndependentStudentSignupForm(
        prefix="independent_student_signup"
    )

    if request.method == "POST":
        if "teacher_signup-teacher_email" in request.POST:
            teacher_signup_form = TeacherSignupForm(
                request.POST, prefix="teacher_signup"
            )

            if not captcha.CAPTCHA_ENABLED:
                remove_captcha_from_forms(teacher_signup_form)

            if teacher_signup_form.is_valid():
                data = teacher_signup_form.cleaned_data
                return process_signup_form(request, data)

        else:
            independent_student_signup_form = IndependentStudentSignupForm(
                request.POST, prefix="independent_student_signup"
            )

            if independent_student_signup_form.is_valid():
                data = independent_student_signup_form.cleaned_data
                return process_independent_student_signup_form(request, data)

    res = render(
        request,
        "portal/register.html",
        {
            "teacher_signup_form": teacher_signup_form,
            "independent_student_signup_form": independent_student_signup_form,
        },
    )

    res.count = invalid_form
    return res


def _newsletter_ticked(form_data):
    return form_data["newsletter_ticked"]


def process_signup_form(request, data):
    email = data["teacher_email"]
    teacher = None

    if email and Teacher.objects.filter(new_user__email=email).exists():
        email_message = email_messages.userAlreadyRegisteredEmail(request, email)
        send_email(
            NOTIFICATION_EMAIL,
            [email],
            email_message["subject"],
            email_message["message"],
        )
    else:
        teacher = Teacher.objects.factory(
            title=data["teacher_title"],
            first_name=data["teacher_first_name"],
            last_name=data["teacher_last_name"],
            email=data["teacher_email"],
            password=data["teacher_password"],
        )

        if _newsletter_ticked(data):
            user = teacher.user.user
            add_to_dotmailer(user.first_name, user.last_name, user.email)

        send_verification_email(request, teacher.user.user)

    if teacher:
        return render(
            request,
            "portal/email_verification_needed.html",
            {"user": teacher.user.user},
        )
    else:
        return render(request, "portal/email_verification_needed.html")


def process_independent_student_signup_form(request, data):
    email = data["email"]
    username = data["username"]

    independent_students = Student.objects.filter(class_field=None)

    if is_independent_email_already_used(email, independent_students):
        email_message = email_messages.userAlreadyRegisteredEmail(
            request, email, is_independent_student=True
        )
        send_email(
            NOTIFICATION_EMAIL,
            [email],
            email_message["subject"],
            email_message["message"],
        )
        return render(request, "portal/email_verification_needed.html")

    if is_independent_username_already_used(username, independent_students):
        email_message = email_messages.indepStudentUsernameAlreadyExistsEmail(
            request, username
        )
        send_email(
            NOTIFICATION_EMAIL,
            [email],
            email_message["subject"],
            email_message["message"],
        )
        return render(request, "portal/email_verification_needed.html")

    student = Student.objects.independentStudentFactory(
        username=data["username"],
        name=data["name"],
        email=data["email"],
        password=data["password"],
    )

    if _newsletter_ticked(data):
        user = student.new_user
        add_to_dotmailer(user.first_name, user.last_name, user.email)

    send_verification_email(request, student.new_user)

    return render(
        request, "portal/email_verification_needed.html", {"user": student.new_user}
    )


def is_independent_email_already_used(email, independent_students):
    return email and independent_students.filter(new_user__email=email).exists()


def is_independent_username_already_used(username, independent_students):
    return (
        username and independent_students.filter(new_user__username=username).exists()
    )


def is_developer(request):
    return hasattr(request.user, "userprofile") and request.user.userprofile.developer


def redirect_teacher_to_correct_page(request, teacher):
    if teacher.has_school():
        classes = teacher.class_teacher.all()
        if classes:
            classes_count = classes.count()
            if classes_count > 1 or classes[0].has_students():
                link = reverse("two_factor:profile")
                messages.info(
                    request,
                    (
                        "You are not currently set up with two-factor authentication. "
                        + "Use your phone or tablet to enhance your account’s security.</br>"
                        + "Click <a href='"
                        + link
                        + "'>here</a> to find out more and "
                        + "set it up or go to your account page at any time."
                    ),
                    extra_tags="safe",
                )
                return reverse_lazy("dashboard")
            else:
                return reverse_lazy(
                    "onboarding-class", kwargs={"access_code": classes[0].access_code},
                )

        else:
            return reverse_lazy("onboarding-classes")
    else:
        return reverse_lazy("onboarding-organisation")


@csrf_exempt
def process_newsletter_form(request):
    if request.method == "POST":
        newsletter_form = NewsletterForm(data=request.POST)
        if newsletter_form.is_valid():
            user_email = newsletter_form.cleaned_data["email"]
            add_to_dotmailer("", "", user_email)
            messages.success(request, "Thank you for signing up! 🎉")
            return HttpResponseRedirect(reverse_lazy("home"))
        messages.error(
            request,
            "Invalid email address. Please try again.",
            extra_tags="sub-nav--warning",
        )
        return HttpResponseRedirect(reverse_lazy("home"))

    return HttpResponse(status=405)


@cache_control(private=True)
def home(request):
    link = reverse("home-learning")
    messages.success(
        request,
        "Families: #KeepKidsCoding! <a href='"
        + link
        + "'>Tap here</a> for free, easy, home coding lessons.",
        extra_tags="safe message__home-learning",
    )

    return render(request, "portal/home.html")


def home_learning(request):
    return render(
        request,
        "portal/home_learning.html",
        {"HOME_LEARNING_BANNER": HOME_LEARNING_BANNER},
    )



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.

import warnings

from django.contrib.auth import get_user_model
from django.contrib.auth.decorators import user_passes_test
from django.contrib.auth.forms import PasswordResetForm, SetPasswordForm
from django.contrib.auth.tokens import default_token_generator
from django.contrib.auth.views import password_reset, password_reset_confirm
from django.core.urlresolvers import reverse_lazy
from django.shortcuts import render
from django.template.response import TemplateResponse
from django.utils.deprecation import RemovedInDjango20Warning
from django.utils.encoding import force_text
from django.utils.http import urlsafe_base64_decode
from django.utils.translation import ugettext as _
from django.views.decorators.cache import never_cache
from django.views.decorators.csrf import csrf_protect
from django.views.decorators.debug import sensitive_post_parameters

from deploy import captcha
from portal import app_settings
from portal.forms.registration import (
    TeacherPasswordResetForm,
    PasswordResetSetPasswordForm,
    StudentPasswordResetForm,
)
from portal.helpers.captcha import remove_captcha_from_form
from common.helpers.emails import PASSWORD_RESET_EMAIL
from common.permissions import not_logged_in, not_fully_logged_in


@user_passes_test(not_logged_in, login_url=reverse_lazy("home"))
def student_password_reset(request):
    usertype = "STUDENT"
    return password_reset(
        request,
        usertype,
        from_email=PASSWORD_RESET_EMAIL,
        template_name="portal/reset_password_student.html",
        password_reset_form=StudentPasswordResetForm,
        is_admin_site=True,
    )


@user_passes_test(not_fully_logged_in, login_url=reverse_lazy("teacher_login"))
def teacher_password_reset(request):
    usertype = "TEACHER"
    return password_reset(
        request,
        usertype,
        from_email=PASSWORD_RESET_EMAIL,
        template_name="portal/reset_password_teach.html",
        password_reset_form=TeacherPasswordResetForm,
        is_admin_site=True,
    )


@csrf_protect
def password_reset(
    request,
    usertype,
    is_admin_site=False,
    template_name="portal/reset_password_teach.html",
    email_template_name="portal/reset_password_email.html",
    subject_template_name="registration/password_reset_subject.txt",
    password_reset_form=PasswordResetForm,
    token_generator=default_token_generator,
    from_email=None,
    current_app=None,
    extra_context=None,
    html_email_template_name=None,
):
    if request.method == "POST":
        form = password_reset_form(request.POST)
        if not captcha.CAPTCHA_ENABLED:
            remove_captcha_from_form(form)
        if form.is_valid():
            opts = {
                "use_https": request.is_secure(),
                "token_generator": token_generator,
                "from_email": from_email,
                "email_template_name": email_template_name,
                "subject_template_name": subject_template_name,
                "request": request,
                "html_email_template_name": html_email_template_name,
            }
            if is_admin_site:
                warnings.warn(
                    "The is_admin_site argument to "
                    "django.contrib.auth.views.password_reset() is deprecated "
                    "and will be removed in Django 2.0.",
                    RemovedInDjango20Warning,
                    3,
                )
                opts = dict(opts, domain_override=request.get_host())
            form.save(**opts)

            return render(
                request, "portal/reset_password_email_sent.html", {"usertype": usertype}
            )
    else:
        form = password_reset_form()

    if not captcha.CAPTCHA_ENABLED:
        remove_captcha_from_form(form)

    context = {
        "form": form,
        "title": _("Password reset"),
        "settings": app_settings,
        "should_use_recaptcha": captcha.CAPTCHA_ENABLED,
    }

    update_context_and_apps(request, context, current_app, extra_context)

    return TemplateResponse(request, template_name, context)


def update_context_and_apps(request, context, current_app, extra_context):
    if extra_context is not None:
        context.update(extra_context)

    if current_app is not None:
        request.current_app = current_app


def password_reset_done(
    request,
    template_name="portal/reset_password_email_sent.html",
    current_app=None,
    extra_context=None,
):
    context = {"title": _("Password reset sent")}

    update_context_and_apps(request, context, current_app, extra_context)

    return TemplateResponse(request, template_name, context)


@sensitive_post_parameters()
@never_cache
def password_reset_confirm(
    request,
    usertype,
    uidb64=None,
    token=None,
    template_name="portal/reset_password_confirm.html",
    token_generator=default_token_generator,
    set_password_form=SetPasswordForm,
    current_app=None,
    extra_context=None,
):
    """
    View that checks the hash in a password reset link and presents a
    form for entering a new password.
    """
    UserModel = get_user_model()
    check_uidb64(uidb64, token)

    try:
        uid = force_text(urlsafe_base64_decode(uidb64))
        user = UserModel._default_manager.get(pk=uid)
    except (TypeError, ValueError, OverflowError, UserModel.DoesNotExist):
        user = None

    if user_is_authenticated(user, token_generator, token):
        validlink = True
        title = _("Enter new password")
        if request.method == "POST":
            form = set_password_form(user, request.POST)
            if form.is_valid():
                form.save()
                return render(
                    request, "portal/reset_password_done.html", {"usertype": usertype}
                )
        else:
            form = set_password_form(user)
    else:
        validlink = False
        form = None
        title = _("Password reset unsuccessful")

    context = {"form": form, "title": title, "validlink": validlink}

    update_context_and_apps(request, context, current_app, extra_context)

    return TemplateResponse(request, template_name, context)


def check_uidb64(uidb64, token):
    assert uidb64 is not None and token is not None  # checked by URLconf


def user_is_authenticated(user, token_generator, token):
    return user is not None and token_generator.check_token(user, token)


@user_passes_test(not_fully_logged_in, login_url=reverse_lazy("home"))
def password_reset_check_and_confirm(request, uidb64=None, token=None):
    """
    Customised standard django auth view with customised form to incorporate checking the password set is strong enough
    """
    UserModel = get_user_model()
    try:
        uid = urlsafe_base64_decode(uidb64)
        user = UserModel._default_manager.get(pk=uid)
    except (TypeError, ValueError, OverflowError, UserModel.DoesNotExist):
        user = None
    if user and hasattr(user, "new_student"):
        usertype = "STUDENT"
    else:
        usertype = "TEACHER"
    return password_reset_confirm(
        request,
        usertype,
        set_password_form=PasswordResetSetPasswordForm,
        uidb64=uidb64,
        token=token,
        extra_context={"usertype": usertype},
    )



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
import time
import uuid
from builtins import str

from common.models import UserProfile
from django.contrib.auth.models import User, Permission
from django.core.urlresolvers import reverse

from portal.tests.base_test import BaseTest
from portal.tests.pageObjects.portal.admin.admin_login_page import AdminLoginPage
from portal.views import admin


class TestAdmin(BaseTest):
    @classmethod
    def setUpClass(cls):
        super(TestAdmin, cls).setUpClass()
        admin.block_limit = 100

    # NB: Users are not expected to navigate to admin login page directly
    def navigate_to_admin_login(self):
        url = self.live_server_url + reverse("administration_login")
        self.selenium.get(url)
        return AdminLoginPage(self.selenium, self.live_server_url)

    def navigate_to_admin_data(self):
        url = self.live_server_url + reverse("aggregated_data")
        self.selenium.get(url)
        # gets redirected to login page when not logged in
        return AdminLoginPage(self.selenium, self.live_server_url)

    def navigate_to_admin_map(self):
        url = self.live_server_url + reverse("map")
        self.selenium.get(url)
        # gets redirected to login page when not logged in
        return AdminLoginPage(self.selenium, self.live_server_url)

    # Checks all admin pages goes to admin_login when user is not logged in
    def test_navigate_to_admin_login(self):
        page = self.navigate_to_admin_login()

    def test_navigate_to_admin_data(self):
        page = self.navigate_to_admin_data()

    def test_navigate_to_admin_map(self):
        page = self.navigate_to_admin_map()

    # Check superuser access to each admin pages
    def test_superuser_access(self):
        username = self.randomId()
        password = "abc123"
        user = User.objects.create_superuser(
            username=username, password=password, email=""
        )
        UserProfile.objects.create(user=user)
        page = self.navigate_to_admin_data().login_to_data(username, password)
        self.assertTrue(page.is_on_admin_data_page())
        page = page.go_to_admin_map_page()
        self.assertTrue(page.is_on_admin_map_page())

    # Check user with view_map_data permission can access to /admin/map but not /admin/data
    def test_view_map_data_permission_access(self):
        username = self.randomId()
        password = "abc123"
        user = User.objects.create_user(username=username, password=password)
        UserProfile.objects.create(user=user)
        permission = Permission.objects.get(codename="view_map_data")
        user.user_permissions.add(permission)
        page = self.navigate_to_admin_map().login_to_map(username, password)
        self.assertTrue(page.is_on_admin_map_page())
        page = page.go_to_admin_data_page_failure()
        self.assertTrue(page.is_on_403_forbidden())

    # Check user with view_aggregated_data permission can access to /admin/data but not /admin/map
    def test_view_aggregated_data_permission_access(self):
        username = self.randomId()
        password = "abc123"
        user = User.objects.create_user(username=username, password=password)
        UserProfile.objects.create(user=user)
        permission = Permission.objects.get(codename="view_aggregated_data")
        user.user_permissions.add(permission)
        page = self.navigate_to_admin_data().login_to_data(username, password)
        self.assertTrue(page.is_on_admin_data_page())
        page = page.go_to_admin_map_page_failure()
        self.assertTrue(page.is_on_403_forbidden())

    def test_no_view_aggregated_data_permission_access(self):
        username = self.randomId()
        password = "abc123"
        User.objects.create_user(username=username, password=password)
        page = self.navigate_to_admin_data().login_to_forbidden(username, password)
        self.assertTrue(page.is_on_403_forbidden())

    def test_no_view_map_data_permission_access(self):
        username = self.randomId()
        password = "abc123"
        User.objects.create_user(username=username, password=password)
        page = self.navigate_to_admin_map().login_to_forbidden(username, password)
        self.assertTrue(page.is_on_403_forbidden())

    def test_wrong_username(self):
        username = self.randomId()
        password = "abc123"
        user = User.objects.create_user(username=username, password=password)
        UserProfile.objects.create(user=user)
        page = self.navigate_to_admin_data().login_failure("user123", password)
        time.sleep(1.5)
        self.assertTrue(page.is_on_admin_login_page())
        self.assertIn(
            "Please enter a correct username and password. Note that both fields may be case-sensitive.",
            page.browser.find_element_by_class_name("errorlist").text,
        )

    def randomId(self):
        return str(uuid.uuid4())



# -*- coding: utf-8 -*-
from django.shortcuts import render

from portal.strings.play_rapid_router import (
    RAPID_ROUTER_BENEFITS,
    RAPID_ROUTER_HEADLINE,
)


def play_rapid_router(request):
    return render(
        request,
        "portal/play_rapid-router.html",
        {"HEADLINE": RAPID_ROUTER_HEADLINE, "BENEFITS": RAPID_ROUTER_BENEFITS},
    )



from django.http import HttpResponse
from django.conf import settings
import os

class BasicAuthMiddleware:
    def unauthed(self):
        response = HttpResponse("""<html><title>Auth required</title><body>
                                <h1>Authorization Required</h1></body></html>""", content_type="text/html")
        response['WWW-Authenticate'] = 'Basic realm="' + os.getenv('APPLICATION_ID', '') + '"'
        response.status_code = 401
        return response

    def process_request(self, request):
        if request.META.has_key('HTTP_AUTHORIZATION'):
            authmeth, auth = request.META['HTTP_AUTHORIZATION'].split(' ', 1)
            if authmeth.lower() == 'basic':
                auth = auth.strip().decode('base64')
                username, password = auth.split(':', 1)
                if username != settings.BASICAUTH_USERNAME or password != settings.BASICAUTH_PASSWORD:
                    return self.unauthed()
            else:
                return self.unauthed()
        else:
            return self.unauthed()



# coding=utf-8
ABOUT_BANNER = {
    "title": "About Code for Life",
    "subtitle": "Code For Life gives everyone the ability to shape technology’s future",
    "text": "",
    "image_class": "banner--picture--about",
}



# -*- coding: utf-8 -*-
from django.shortcuts import render

from portal.strings.about import ABOUT_BANNER


def about(request):
    return render(request, "portal/about.html", {"BANNER": ABOUT_BANNER})



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from django.core import mail
from django.core.urlresolvers import reverse
from django.test import Client
import pytest

from common.helpers.emails import add_to_dotmailer


@pytest.mark.django_db
def test_send_new_users_numbers_email():
    client = Client()
    response = client.get(reverse("send_new_users_report"))
    assert response.status_code == 200
    assert len(mail.outbox) == 1


def test_newsletter_sends_correct_request(mocker, monkeypatch):
    mocked_post = mocker.patch("common.helpers.emails.post")

    expected_body = {
        "email": "ray.charles@example.com",
        "optInType": "VerifiedDouble",
        "emailType": "Html",
        "dataFields": [
            {"key": "FIRSTNAME", "value": "Ray"},
            {"key": "LASTNAME", "value": "Charles"},
            {"key": "FULLNAME", "value": "Ray Charles"},
        ],
        "preferences": [{"trout": True}],
    }

    add_to_dotmailer("Ray", "Charles", "ray.charles@example.com")

    mocked_post.assert_called_once_with(
        "https://test/", auth=("username_here", "password_here"), json=expected_body
    )



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from portal.tests.pageObjects.portal.admin.admin_base_page import AdminBasePage


class AdminLoginPage(AdminBasePage):
    def __init__(self, browser, live_server_url):
        super(AdminLoginPage, self).__init__(browser, live_server_url)

        assert self.on_correct_page("administration_login")

    def login_to_forbidden(self, username, password):
        self._login(username, password)

        from portal.tests.pageObjects.portal.forbidden_page import ForbiddenPage

        return ForbiddenPage(self.browser)

    def login_failure(self, username, password):
        self._login(username, password)
        return self

    def login_to_data(self, username, password):
        self._login(username, password)
        from portal.tests.pageObjects.portal.admin.admin_data_page import AdminDataPage

        return AdminDataPage(self.browser, self.live_server_url)

    def login_to_map(self, username, password):
        self._login(username, password)
        from portal.tests.pageObjects.portal.admin.admin_map_page import AdminMapPage

        return AdminMapPage(self.browser, self.live_server_url)

    def _login(self, username, password):
        id_username_field = self.browser.find_element_by_id("id_username")
        id_password_field = self.browser.find_element_by_id("id_password")
        login_field = self.browser.find_element_by_name("login_view")
        id_username_field.clear()
        id_password_field.clear()
        id_username_field.send_keys(username)
        id_password_field.send_keys(password)
        login_field.click()



# -*- coding: utf-8 -*-

from django.contrib.auth.decorators import login_required, user_passes_test
from django.core.urlresolvers import reverse_lazy
from django.shortcuts import render

from common.permissions import logged_in_as_teacher
from portal.strings.teacher_resources import RESOURCES_BANNER


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def teacher_resources(request):
    return render(
        request, "portal/teach/teacher_resources.html", {"BANNER": RESOURCES_BANNER}
    )



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from __future__ import absolute_import

from builtins import str

from django.contrib.auth.models import User
from django.core.urlresolvers import reverse, reverse_lazy
from django.test import Client, TestCase

from common.models import School, Student, UserProfile
from common.tests.utils.classes import create_class_directly
from common.tests.utils.teacher import signup_teacher_directly


class SecurityTestCase(TestCase):
    def _test_incorrect_teacher_cannot_login(self, view_name):
        email1, _ = signup_teacher_directly()
        email2, pass2 = signup_teacher_directly()
        _, _, access_code = create_class_directly(email1)

        c = Client()
        assert c.login(username=email2, password=pass2)
        page = reverse(view_name, args=[access_code])
        self.assertNotEqual(c.get(page).status_code, 200)

    def _test_incorrect_teacher_no_info_leak(self, view_name):
        email1, _ = signup_teacher_directly()
        email2, pass2 = signup_teacher_directly()
        _, _, access_code = create_class_directly(email1)

        c = Client()
        assert c.login(username=email2, password=pass2)

        invalid_page = reverse(view_name, args=[access_code])
        invalid_login_code = c.get(invalid_page).status_code

        non_existant_page = reverse(view_name, args=["AAAAA"])
        non_existant_code = c.get(non_existant_page).status_code

        self.assertEqual(non_existant_code, invalid_login_code)

    def test_reminder_cards_info_leak(self):
        """Check that it isn't leaked whether an access code exists."""
        self._test_incorrect_teacher_no_info_leak("teacher_print_reminder_cards")

    def test_class_page_info_leak(self):
        """Check that it isn't leaked whether an access code exists."""
        self._test_incorrect_teacher_no_info_leak("onboarding-class")

    def test_student_edit_info_leak(self):
        c = Client()
        t_email, t_pass = signup_teacher_directly()
        c.login(email=t_email, password=t_pass)
        profile = UserProfile(user=User.objects.create_user("test"))
        profile.save()
        stu = Student(user=profile)
        stu.save()

        self.assertEqual(
            c.get(reverse("teacher_edit_student", kwargs={"pk": "9999"})).status_code,
            c.get(reverse("teacher_edit_student", kwargs={"pk": stu.pk})).status_code,
        )

    def test_cannot_lookup_schools_if_not_logged_in(self):
        client = Client()

        url = reverse("organisation_fuzzy_lookup")
        data = {"fuzzy_name": ["A"]}
        response = client.get(url, data=data)

        self.assertEqual(403, response.status_code)

    def test_cannot_create_school_with_email_as_name(self):
        number_of_existing_schools = len(School.objects.all())

        email, password = signup_teacher_directly()

        client = Client()
        client.login(username=email, password=password)

        url = reverse("onboarding-organisation")
        data = {
            "name": email,
            "postcode": "TEST",
            "country": "GB",
            "create_organisation": "",
        }

        client.post(url, data)

        self.assertEqual(number_of_existing_schools, len(School.objects.all()))

    def test_reminder_cards_wrong_teacher(self):
        """Try and view reminder cards without being the teacher for that class."""
        self._test_incorrect_teacher_cannot_login("teacher_print_reminder_cards")

    def test_class_page_wrong_teacher(self):
        """Try and view a class page without being the teacher for that class."""
        self._test_incorrect_teacher_cannot_login("onboarding-class")

    def test_anonymous_cannot_access_teaching_materials(self):
        c = Client()
        page = reverse_lazy("materials")
        self.assertNotEqual(str(c.get(page).status_code)[0], 2)



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from django.contrib import admin
from django.contrib.auth.admin import UserAdmin
from django.contrib.auth.models import User

from common.models import (
    Class,
    EmailVerification,
    School,
    Student,
    Teacher,
    UserProfile,
)
from portal.models import Guardian


class ClassAdmin(admin.ModelAdmin):
    search_fields = [
        "name",
        "teacher__new_user__first_name",
        "teacher__new_user__last_name",
        "teacher__school__name",
    ]
    list_display = ["__str__", "teacher", "teacher_school"]
    raw_id_fields = ["teacher"]

    def teacher_school(self, obj):
        return obj.teacher.school


class SchoolAdmin(admin.ModelAdmin):
    search_fields = ["name", "country", "postcode", "town"]
    list_filter = ["postcode", "country"]


class StudentAdmin(admin.ModelAdmin):
    search_fields = [
        "new_user__first_name",
        "new_user__last_name",
        "class_field__name",
        "class_field__teacher__new_user__first_name",
        "class_field__teacher__new_user__last_name",
        "class_field__teacher__school__name",
    ]
    list_display = [
        "__str__",
        "class_field",
        "class_field_teacher",
        "class_field_school",
    ]
    readonly_fields = ["user", "new_user"]
    raw_id_fields = ["class_field", "pending_class_request"]

    def class_field_teacher(self, obj):
        if obj.class_field:
            return obj.class_field.teacher
        else:
            return "Independent"

    def class_field_school(self, obj):
        if obj.class_field:
            return obj.class_field.teacher.school
        else:
            return "Independent"


class TeacherAdmin(admin.ModelAdmin):
    search_fields = ["new_user__first_name", "new_user__last_name", "school__name"]
    list_display = ["__str__", "school"]
    readonly_fields = ["user", "new_user"]
    raw_id_fields = ["school", "pending_join_request"]


class UserProfileAdmin(admin.ModelAdmin):
    search_fields = [
        "user__first_name",
        "user__last_name",
        "user__username",
        "user__date_joined",
    ]
    list_filter = ["user__date_joined"]
    list_display = ["user", "__str__", "joined_recently"]
    readonly_fields = ["user"]


class EmailVerificationAdmin(admin.ModelAdmin):
    search_fields = [
        "user__first_name",
        "user__last_name",
        "user__username",
        "user__date_joined",
    ]
    readonly_fields = ["user", "token"]


UserAdmin.list_display += ("date_joined",)
UserAdmin.list_filter += ("date_joined",)


admin.site.register(Class, ClassAdmin)
admin.site.register(Student, StudentAdmin)
admin.site.register(Guardian)
admin.site.register(Teacher, TeacherAdmin)
admin.site.register(School, SchoolAdmin)
admin.site.unregister(User)
admin.site.register(User, UserAdmin)
admin.site.register(UserProfile, UserProfileAdmin)
admin.site.register(EmailVerification, EmailVerificationAdmin)



# -*- coding: utf-8 -*-
from django.shortcuts import render

from portal import app_settings
from portal.strings.help_and_support import HELP_BANNER


def contact(request):
    response = render(
        request,
        "portal/help-and-support.html",
        {"settings": app_settings, "BANNER": HELP_BANNER},
    )

    return response



from .teacher import TeacherLoginView
from .independent_student import IndependentStudentLoginView
from .student import StudentLoginView

from django.shortcuts import redirect
from django.core.urlresolvers import reverse_lazy


def old_login_form_redirect(request):
    return redirect(reverse_lazy("home"))
